from typing import List
from typing import Optional
from typing import Tuple

import attr
import pendulum
from pyspark.sql import DataFrame
from pyspark.sql import functions
from pyspark.sql import SparkSession

from tecton_proto.data.new_transformation_pb2 import NewTransformation as Transformation
from tecton_proto.data.virtual_data_source_pb2 import VirtualDataSource
from tecton_spark.feature_definition_wrapper import FeatureDefinitionWrapper as FeatureDefinition
from tecton_spark.partial_aggregations import construct_partial_time_aggregation_df
from tecton_spark.partial_aggregations import TEMPORAL_ANCHOR_COLUMN_NAME
from tecton_spark.pipeline_helper import pipeline_to_dataframe
from tecton_spark.spark_time_utils import convert_timestamp_to_epoch
from tecton_spark.time_utils import convert_timedelta_for_version

MATERIALIZED_RAW_DATA_END_TIME = "_materialized_raw_data_end_time"
TECTON_FEATURE_TIMESTAMP_VALIDATOR = "_tecton_feature_timestamp_validator"


@attr.s(auto_attribs=True)
class MaterializationPlan(object):
    offline_store_data_frame: Optional[DataFrame]
    online_store_data_frame: Optional[DataFrame]
    # should be the most recent ancestor of both online and offline so we can cache both of them easily
    base_data_frame: Optional[DataFrame]
    coalesce: int


def get_batch_materialization_plan(
    *,
    spark: SparkSession,
    feature_definition: FeatureDefinition,
    feature_data_time_limits: Optional[pendulum.Period],
    coalesce: int,
    data_sources: List[VirtualDataSource],
    called_for_online_feature_store: bool = False,
    transformations: Optional[List[Transformation]] = None,
    schedule_interval: Optional[pendulum.Duration] = None,
) -> MaterializationPlan:
    """
    NOTE: We rely on Spark's lazy evaluation model to infer partially materialized tile Schema during FeatureView
    creation time without actually performing any materialization.
    Please make sure to not perform any Spark operations under this function's code path that will actually execute
    the Spark query (e.g: df.count(), df.show(), etc.).
    """

    if feature_definition.is_temporal_aggregate:
        return _get_batch_materialization_plan_for_aggregate_feature_view(
            spark,
            feature_definition,
            False,
            feature_data_time_limits,
            data_sources,
            transformations or [],
            coalesce,
            schedule_interval=schedule_interval,
        )
    elif feature_definition.is_temporal:
        assert feature_data_time_limits is not None
        return _get_batch_materialization_plan_for_temporal_feature_view(
            spark,
            feature_definition,
            False,
            feature_data_time_limits,
            data_sources,
            transformations or [],
            coalesce,
            called_for_online_feature_store,
            schedule_interval=schedule_interval,
        )
    else:
        raise ValueError(f"Unhandled feature view: {feature_definition.fv}")


def _get_batch_materialization_plan_for_aggregate_feature_view(
    spark: SparkSession,
    feature_definition: FeatureDefinition,
    consume_streaming_data_sources: bool,
    feature_data_time_limits: Optional[pendulum.Period],
    data_sources: List[VirtualDataSource],
    transformations: List[Transformation],
    coalesce: int,
    schedule_interval: Optional[pendulum.Duration] = None,
) -> MaterializationPlan:
    df = pipeline_to_dataframe(
        spark,
        feature_definition.fv.pipeline,
        consume_streaming_data_sources,
        data_sources,
        transformations,
        feature_time_limits=feature_data_time_limits,
        schedule_interval=schedule_interval,
    )
    timestamp_key = feature_definition.timestamp_key
    spark_df = _validate_feature_timestamps(spark, df, feature_data_time_limits, timestamp_key)

    trailing_time_window_aggregation = feature_definition.trailing_time_window_aggregation
    online_store_df = offline_store_df = underlying_df = construct_partial_time_aggregation_df(
        spark_df,
        list(feature_definition.join_keys),
        trailing_time_window_aggregation,
        feature_definition.get_feature_store_format_version,
    )

    return MaterializationPlan(offline_store_df, online_store_df, underlying_df, coalesce)


def _get_batch_materialization_plan_for_temporal_feature_view(
    spark: SparkSession,
    feature_definition: FeatureDefinition,
    consume_streaming_data_sources: bool,
    feature_data_time_limits: pendulum.Period,
    data_sources: List[VirtualDataSource],
    transformations: List[Transformation],
    coalesce: int,
    called_for_online_feature_store: bool,
    schedule_interval: Optional[pendulum.Duration] = None,
) -> MaterializationPlan:
    offline_store_df, online_store_df, underlying_df = _materialize_interval_for_temporal_feature_view(
        spark,
        feature_definition,
        feature_data_time_limits,
        data_sources,
        transformations,
        called_for_online_feature_store,
        consume_streaming_data_sources,
        schedule_interval=schedule_interval,
    )

    if called_for_online_feature_store:
        serving_ttl = feature_definition.serving_ttl
        if serving_ttl.in_seconds() > 0:
            online_store_df = _possibly_limit_online_store_df(
                online_store_df, feature_definition.timestamp_key, feature_data_time_limits.end, serving_ttl
            )

    return MaterializationPlan(offline_store_df, online_store_df, underlying_df, coalesce)


def _materialize_interval_for_temporal_feature_view(
    spark: SparkSession,
    fd: FeatureDefinition,
    feature_data_time_limits: pendulum.Period,
    data_sources: List[VirtualDataSource],
    transformations: List[Transformation],
    called_for_online_feature_store: bool,
    consume_streaming_data_sources: bool,
    schedule_interval: Optional[pendulum.Duration] = None,
) -> Tuple[DataFrame, DataFrame, DataFrame]:
    tile_df = pipeline_to_dataframe(
        spark,
        fd.pipeline,
        consume_streaming_data_sources,
        data_sources,
        transformations,
        feature_time_limits=feature_data_time_limits,
        schedule_interval=schedule_interval,
    )

    timestamp_key = fd.timestamp_key
    tile_df = _validate_feature_timestamps(spark, tile_df, feature_data_time_limits, timestamp_key)
    cacheable_df = tile_df

    # We infer partition column (i.e. anchor time) by looking at the feature timestamp column and grouping
    # all the features within `[anchor_time,  anchor_time + batch_schedule)` together.
    version = fd.get_feature_store_format_version
    anchor_time_val = convert_timestamp_to_epoch(functions.col(timestamp_key), version)
    batch_mat_schedule = convert_timedelta_for_version(fd.batch_materialization_schedule, version)
    offline_store_tile_df = tile_df.withColumn(
        TEMPORAL_ANCHOR_COLUMN_NAME, anchor_time_val - anchor_time_val % batch_mat_schedule
    )

    if called_for_online_feature_store:
        # Add raw data end time as a column as it's used for status reporting while writing to Online Feature Store.
        # When materializing multiple tiles, include `raw_data_end_time` per tile so that we can distribute writing
        # to Kafka into multiple partitions.
        online_store_tile_df = offline_store_tile_df.withColumn(
            MATERIALIZED_RAW_DATA_END_TIME, functions.col(TEMPORAL_ANCHOR_COLUMN_NAME) + batch_mat_schedule
        ).drop(TEMPORAL_ANCHOR_COLUMN_NAME)
    else:
        online_store_tile_df = tile_df

    return offline_store_tile_df, online_store_tile_df, cacheable_df


def _validate_feature_timestamps(
    spark: SparkSession,
    feature_df: DataFrame,
    feature_data_time_limits: Optional[pendulum.Period],
    timestamp_key: Optional[str],
) -> DataFrame:
    if feature_data_time_limits:
        # Register the Spark UDF that is contained in the JAR files and that is part of passed Spark session.
        # Jars are included the following way into the Spark session:
        #  - For materialization jobs scheduled by Orchestrator, they are included in the Job submission API.
        #    In this case, we always use the default Spark session of the spun-up Spark cluster.
        #  - For interactive execution (or remote over db-connect / livy), we always construct Spark session
        #    manually and include appropriate JARs ourselves.
        udf_generator = spark.sparkContext._jvm.com.tecton.udfs.spark3.RegisterFeatureTimestampValidator()
        udf_generator.register(TECTON_FEATURE_TIMESTAMP_VALIDATOR)

        start_time_expr = f"to_timestamp('{feature_data_time_limits.start}')"
        # Registered feature timestamp validation UDF checks that each timestamp is within *closed* time interval: [start_time, end_time].
        # So we subtract 1 microsecond here, before passing time limits to the UDF.
        end_time_expr = f"to_timestamp('{feature_data_time_limits.end - pendulum.duration(microseconds=1)}')"
        filter_expr = f"{TECTON_FEATURE_TIMESTAMP_VALIDATOR}({timestamp_key}, {start_time_expr}, {end_time_expr}, '{timestamp_key}')"

        # Force the output of the UDF to be filtered on, so the UDF cannot be optimized away.
        feature_df = feature_df.where(filter_expr)

    return feature_df


def _possibly_limit_online_store_df(
    online_df: DataFrame,
    timestamp_column_name: str,
    feature_end_time: pendulum.DateTime,
    timestamp_range_needed: pendulum.Duration,
) -> DataFrame:
    earliest_timestamp_needed = feature_end_time - timestamp_range_needed
    return online_df.where(functions.col(timestamp_column_name) >= earliest_timestamp_needed)


def get_stream_materialization_plan(
    spark: SparkSession,
    data_sources: List[VirtualDataSource],
    transformations: List[Transformation],
    feature_definition: FeatureDefinition,
) -> MaterializationPlan:
    transformations = transformations or []

    df = None
    if feature_definition:
        df = pipeline_to_dataframe(spark, feature_definition.pipeline, True, data_sources, transformations)
        if feature_definition.is_temporal_aggregate:
            df = construct_partial_time_aggregation_df(
                df,
                list(feature_definition.join_keys),
                feature_definition.trailing_time_window_aggregation,
                feature_definition.get_feature_store_format_version,
            )

    return MaterializationPlan(None, df, df, 0)



from typing import List

from tecton_proto.args.basic_info_pb2 import BasicInfo
from tecton_proto.args.entity_pb2 import EntityArgs
from tecton_spark.id_helper import IdHelper


def prepare_args(*, basic_info: BasicInfo, join_keys: List[str]) -> EntityArgs:
    args = EntityArgs()
    args.entity_id.CopyFrom(IdHelper.from_string(IdHelper.generate_string_id()))
    args.info.CopyFrom(basic_info)
    args.join_keys.extend(join_keys)
    return args



import sys
from dataclasses import dataclass
from pathlib import Path
from typing import List
from typing import Optional
from typing import Set


@dataclass
class RepoData:
    # paths and file set are different representations of the same data
    paths: List[Path] = None
    file_set: Set[str] = None
    root: str = None

    @property
    def initialized(self):
        return self.paths is not None


_repo_data = RepoData()

# Prepare a repo for tecton objects to be processed.
# If file_in_repo is not set, expects the current directory to be inside the feature repo.
def ensure_prepare_repo(file_in_repo: Optional[str] = None):
    if _repo_data.initialized:
        # repo is already prepared
        return
    root = _maybe_get_repo_root(file_in_repo)
    if root is None:
        raise Exception(f"Feature repository root not found. Run `tecton init` to set it.")
    paths = get_repo_files(root)
    file_set = {str(f) for f in paths}
    _repo_data.paths, _repo_data.file_set, _repo_data.root = paths, file_set, root


def repo_files() -> List[Path]:
    if not _repo_data.initialized:
        raise Exception("Repo is not prepared")
    return _repo_data.paths


def repo_files_set() -> Set[str]:
    if not _repo_data.initialized:
        raise Exception("Repo is not prepared")
    return _repo_data.file_set


def repo_root() -> str:
    if not _repo_data.initialized:
        raise Exception("Repo is not prepared")
    return _repo_data.root


def _fake_init_for_testing(root: str = ""):
    fake_root = Path(root)
    _repo_data.paths, _repo_data.file_set, _repo_data.root = [fake_root], {str(fake_root)}, fake_root


# Finds the repo root of a given python file (a parent directory containing ".tecton")
# If a file is not passed in, searches the current directory's parents.
# Returns None if the root is not found
def _maybe_get_repo_root(file_in_repo: Optional[str] = None) -> Optional[str]:
    if _repo_data.root is not None:
        return _repo_data.root

    if file_in_repo is None:
        d = Path().resolve()
    else:
        d = Path(file_in_repo)
    while d.parent != d and d != Path.home():
        tecton_cfg = d / Path(".tecton")
        if tecton_cfg.exists():
            _repo_data.root = str(d)
            return _repo_data.root
        d = d.parent
    return None


def _get_ignored_files(repo_root: Path) -> Set[Path]:
    import pathspec

    ignorefile = repo_root / Path(".tectonignore")
    if not ignorefile.exists():
        return set()

    with open(ignorefile, "r") as f:
        spec = pathspec.PathSpec.from_lines("gitwildmatch", f)
        return {repo_root / file_name for file_name in spec.match_tree(str(repo_root))}


def get_repo_files(root: str, suffixes=[".py", ".yml", "yaml"]) -> List[Path]:
    root_path = Path(root)
    repo_files = [p.resolve() for p in root_path.glob("**/*") if p.suffix in suffixes]

    # Ignore virtualenv directory if any, typically you'd have /some/path/bin/python as an
    # interpreter, so we want to skip anything under /some/path/
    if sys.executable:
        python_dir = Path(sys.executable).parent.parent

        # we might be dealing with virtualenv
        if root_path.resolve() in python_dir.parents:
            repo_files = [p for p in repo_files if python_dir not in p.parents]

    # Filter out files under hidden dirs starting with "/." for PosixPath or "\." for WindowsPath
    repo_files = list(filter(lambda p: "/." not in str(p) and "\." not in str(p), repo_files))

    # Filter out files that match glob expressions in .tectonignore
    ignored_files = _get_ignored_files(root_path)
    filtered_files = [p for p in repo_files if p not in ignored_files]

    return filtered_files



import py4j

from tecton_spark.errors import AccessError


def handleDataAccessErrors(func, details):
    try:
        return func()
    except Exception as e:
        if isinstance(
            e, py4j.protocol.Py4JJavaError
        ) and "com.amazonaws.services.s3.model.AmazonS3Exception: Forbidden" in str(e):
            raise AccessError(f"Unable to access file: {details}") from e
        raise e



_ALL_FCOS = {}

# An internal superclass of declarative Tecton Objects
class Fco:
    @classmethod
    def _register(cls, fco):
        _ALL_FCOS[fco._id] = fco

    @property  # type: ignore
    def _id(self) -> str:
        """
        The id of this Tecton Object.
        """
        raise NotImplementedError()



from typing import Dict
from typing import List
from typing import Optional
from typing import Union

from tecton import LoggingConfig
from tecton._internals.fco import Fco
from tecton._internals.feature_definition import FeatureDefinition
from tecton.basic_info import prepare_basic_info
from tecton.feature_services.feature_service_args import FeaturesConfig
from tecton.feature_services.feature_service_args import prepare_args
from tecton_proto.args.feature_service_pb2 import FeatureServiceArgs
from tecton_proto.args.repo_metadata_pb2 import SourceInfo
from tecton_proto.common.id_pb2 import Id
from tecton_spark.logger import get_logger

logger = get_logger("FeatureService")


class FeatureService(Fco):
    """
    Declare a FeatureService.

    In Tecton, a Feature Service exposes an API for accessing a set of FeatureViews.
    FeatureServices are implemented using the ``FeatureService`` class.

    Once deployed in production, each model has one associated Feature Service that
    serves the model its features. A Feature Service contains a list of the Feature
    Views associated with a model. It also includes user-provided metadata such as
    name, description, and owner that Tecton uses to organize feature data.
    """

    _args: FeatureServiceArgs
    _source_info: SourceInfo

    def __init__(
        self,
        *,
        name: str,
        description: str = "",
        family: str = "",
        tags: Dict[str, str] = None,
        owner: str = "",
        online_serving_enabled: bool = True,
        features: List[Union[FeaturesConfig, FeatureDefinition]] = None,
        logging: Optional[LoggingConfig] = None,
    ):
        """
        Instantiates a new FeatureService.

        :param name: A unique name for the Feature Service.
        :param description: (Optional) description.
        :param family: (Optional) Family of this Feature Service, used to group Tecton Objects.
        :param tags: (Optional) Tags associated with this Tecton Object (key-value pairs of arbitrary metadata).
        :param owner: Owner name (typically the email of the primary maintainer).
        :param online_serving_enabled: (Optional, default True) If True, users can send realtime requests
            to this FeatureService, and only FeatureViews with online materialization enabled can be added
            to this FeatureService.
        :param features: The list of FeatureView or FeaturesConfig that this FeatureService will serve.
        :param logging: (Optional) A configuration for logging feature requests sent to this Feature Service.

        An example of Feature Service declaration

        .. code-block:: python

            from tecton import FeatureService, LoggingConfig
            # Import your feature views declared in your feature repo directory
            from feature_repo.features.feature_views import last_transaction_amount_sql, transaction_amount_is_high
            ...

            # Declare Feature Service
            fraud_detection_feature_service = FeatureService(
                name='fraud_detection_feature_service',
                description='A FeatureService providing features for a model that predicts if a transaction is fraudulent.',
                family='fraud',
                features=[
                    last_transaction_amount_sql,
                    transaction_amount_is_high,
                    ...
                ]
                logging=LoggingConfig(
                    sample_rate=0.5,
                    log_effective_time=False,
                )
                tags={'release': 'staging'},
            )
        """
        from tecton.cli.common import get_fco_source_info

        basic_info = prepare_basic_info(name=name, description=description, owner=owner, family=family, tags=tags)

        self._source_info = get_fco_source_info()
        self._args = prepare_args(
            basic_info=basic_info,
            online_serving_enabled=online_serving_enabled,
            features=features or [],
            logging=logging,
        )

        Fco._register(self)

    def _id(self) -> Id:
        return self._args.feature_service_id

    @property
    def name(self) -> str:
        """
        Name of this FeatureService.
        """
        return self._args.info.name



from typing import Dict
from typing import List
from typing import Optional
from typing import Union

import attr
import pendulum
import pytimeparse
from google.protobuf.duration_pb2 import Duration

from tecton._internals import errors
from tecton.aggregation_functions import AggregationFunction
from tecton_proto.args import feature_view_pb2

AVAILABILITY_SPOT = "spot"
AVAILABILITY_ON_DEMAND = "on_demand"
AVAILABILITY_SPOT_FALLBACK = "spot_with_fallback"
DATABRICKS_SUPPORTED_AVAILABILITY = [AVAILABILITY_SPOT, AVAILABILITY_ON_DEMAND, AVAILABILITY_SPOT_FALLBACK]
EMR_SUPPORTED_AVAILABILITY = [AVAILABILITY_SPOT, AVAILABILITY_ON_DEMAND]
BACKFILL_CONFIG_MODE_PROTO_PREFIX = "BACKFILL_CONFIG_MODE_"
BACKFILL_CONFIG_MODE_MULTIPLE = "multiple_batch_schedule_intervals_per_job"
BACKFILL_CONFIG_MODE_SINGLE = "single_batch_schedule_interval_per_job"


@attr.s(auto_attribs=True)
class FeatureAggregation(object):
    """
    This class describes a single aggregation that is applied in a batch or stream window aggregate feature view.

    :param column: Column name of the feature we are aggregating.
    :type column: str
    :param function: One of the built-in aggregation functions.
    :type function: Union[str, AggregationFunction]
    :param time_windows: Duration to aggregate over in pytimeparse_ format. Examples: ``"30days"``, ``["8hours", "30days", "365days"]``.
    :type time_windows: Union[str, List[str]]

    `function` can be one of predefined numeric aggregation functions, namely ``"count"``, ``"sum"``, ``"mean"``, ``"min"``, ``"max"``. For
    these numeric aggregations, you can pass the name of it as a string. Nulls are handled like Spark SQL `Function(column)`, e.g. SUM/MEAN/MIN/MAX of all nulls is null and COUNT of all nulls is 0.

    In addition to numeric aggregations, :class:`FeatureAggregation` supports "last-n" aggregations that
    will compute the last N distinct values for the column by timestamp. Right now only string column types are supported as inputs
    to this aggregation, i.e., the resulting feature value will be a list of strings. Nulls are not included in the aggregated list.

    You can use it via the ``last_distinct()`` helper function like this:

    .. code-block:: python

        from tecton.aggregation_functions import last_distinct
        my_fv = BatchWindowAggregateFeatureView(
        ...
        aggregations=[FeatureAggregation(
            column='my_column',
            function=last_distinct(15),
            time_windows=['7days'])],
        ...
        )

    .. _pytimeparse: https://pypi.org/project/pytimeparse/
    """

    column: str
    """"""
    function: Union[str, AggregationFunction]
    """One of the built-in aggregation functions (`'count'`, `'sum'`, `'mean'`, `'min'`, `'max'`)."""
    time_windows: Union[str, List[str]]
    """
       Examples: `"30days"`, `["8hours", "30days", "365days"]`.

       .. _pytimeparse: https://pypi.org/project/pytimeparse/
       """

    def _to_proto(self):
        proto = feature_view_pb2.FeatureAggregation()
        proto.column = self.column

        if isinstance(self.function, str):
            proto.function = self.function
        elif isinstance(self.function, AggregationFunction):
            proto.function = self.function.name
            for k, v in self.function.params.items():
                assert isinstance(v, int)
                proto.function_params[k].CopyFrom(feature_view_pb2.ParamValue(int64_value=v))
        else:
            raise TypeError(f"Invalid function type: {type(self.function)}")

        windows = self.time_windows if isinstance(self.time_windows, list) else [self.time_windows]
        for w in windows:
            duration = Duration()
            duration.FromTimedelta(pendulum.duration(seconds=pytimeparse.parse(w)))
            proto.time_window_durations.append(duration)
            proto.time_windows.append(w)
        return proto


@attr.s(auto_attribs=True)
class ExistingClusterConfig:
    """Use an existing Databricks cluster.

    :param existing_cluster_id: ID of the existing cluster.
    """

    existing_cluster_id: str

    def _to_proto(self) -> feature_view_pb2.ExistingClusterConfig:
        proto = feature_view_pb2.ExistingClusterConfig()
        proto.existing_cluster_id = self.existing_cluster_id

        return proto

    def _to_cluster_proto(self) -> feature_view_pb2.ClusterConfig:
        return feature_view_pb2.ClusterConfig(existing_cluster=self._to_proto())


@attr.s(auto_attribs=True)
class EMRClusterConfig:
    """Configuration used to specify materialization cluster options.

    This class describes the attributes of the new clusters which are created in EMR during
    materialization jobs. You can configure options of these clusters, like cluster size and extra pip dependencies.

    :param instance_type: Instance type for the cluster. Must be a valid type as listed in https://docs.aws.amazon.com/emr/latest/ManagementGuide/emr-supported-instance-types.html.
        If not specified, a value determined by the Tecton backend is used.
    :param instance_availability: Instance availability for the cluster : "spot" or "on_demand".
        If not specified, default is spot.
    :param number_of_workers: Number of instances for the materialization job. If not specified, a value determined by the Tecton backend is used
    :param first_on_demand: The first `first_on_demand` nodes of the cluster will use on_demand instances. The rest will use the type specified by instance_availability.
        If first_on_demand >= 1, the master node will use on_demand instance type. `first_on_demand` is recommended to be set >= 1 for cluster configs for critical streaming features.
    :param root_volume_size_in_gb: Size of the root volume in GB per instance for the materialization job.
        If not specified, a value determined by the Tecton backend is used.
    :param extra_pip_dependencies: Extra pip dependencies to be installed on the cluster.
    :param spark_config: Map of Spark configuration options and their respective values that will be passed to the
        FeatureView materialization Spark cluster. Currently, we support only the following options: ``spark.driver.memory``, ``spark.driver.memoryOverhead``, ``spark.executor.memory``, ``spark.executor.memoryOverhead``

    Note on ``extra_pip_dependencies``: This is a list of pip package names that will be installed during
    materialization. These libraries will only be available to use inside Spark UDFs. For example, if you set
    ``extra_pip_dependencies=["tensorflow"]``, you can use it in your transformation as shown below.

    An example of EMRClusterConfig.

    .. code-block:: python

        from tecton import batch_feature_view, Input, EMRClusterConfig

        @batch_feature_view(
            inputs={'credit_scores': Input(credit_scores_batch)},
            # Can be an argument instance to a batch feature view decorator
            batch_cluster_config = EMRClusterConfig(
                instance_type = 'm5.2xlarge',
                number_of_workers=4,
                extra_pip_dependencies=["tensorflow==2.2.0"],
            ),
            # Other named arguments to batch feature view
            ...
        )

        # Use the tensorflow package in the UDF since tensorflow will be installed
        # on the EMR Spark cluster. The import has to be within the UDF body. Putting it at the
        # top of the file or inside transformation function won't work.

        @transformation(mode='pyspark')
        def test_transformation(transformation_input):
            from pyspark.sql import functions as F
            from pyspark.sql.types import IntegerType

            def my_tensorflow(x):
                import tensorflow as tf
                return int(tf.math.log1p(float(x)).numpy())

            my_tensorflow_udf = F.udf(my_tensorflow, IntegerType())

            return transformation_input.select(
                'entity_id',
                'timestamp',
                my_tensorflow_udf('clicks').alias('log1p_clicks')
            )
    """

    instance_type: Optional[str] = None
    instance_availability: Optional[str] = None
    number_of_workers: Optional[int] = None
    first_on_demand: Optional[int] = None
    root_volume_size_in_gb: Optional[int] = None
    extra_pip_dependencies: Optional[List[str]] = None
    spark_config: Optional[Dict[str, str]] = None

    def _to_proto(self) -> feature_view_pb2.NewEMRClusterConfig:
        proto = feature_view_pb2.NewEMRClusterConfig()
        if self.instance_type:
            proto.instance_type = self.instance_type
        if self.instance_availability:
            if self.instance_availability not in EMR_SUPPORTED_AVAILABILITY:
                raise ValueError(
                    f"Instance availability {self.instance_availability} is not supported. Choose {AVAILABILITY_SPOT} or {AVAILABILITY_ON_DEMAND}"
                )
            proto.instance_availability = self.instance_availability
        if self.number_of_workers:
            proto.number_of_workers = self.number_of_workers
        if self.first_on_demand:
            proto.first_on_demand = self.first_on_demand
        if self.root_volume_size_in_gb:
            proto.root_volume_size_in_gb = self.root_volume_size_in_gb
        if self.extra_pip_dependencies:
            proto.extra_pip_dependencies.extend(self.extra_pip_dependencies)
        if self.spark_config:
            spark_config = SparkConfigWrapper(self.spark_config)._to_proto()
            proto.spark_config.CopyFrom(spark_config)

        return proto

    def _to_cluster_proto(self) -> feature_view_pb2.ClusterConfig:
        return feature_view_pb2.ClusterConfig(new_emr=self._to_proto())


@attr.s(auto_attribs=True)
class DatabricksClusterConfig:
    """Configuration used to specify materialization cluster options.

    This class describes the attributes of the new clusters which are created in Databricks during
    materialization jobs. You can configure options of these clusters, like cluster size and extra pip dependencies.

    :param instance_type: Instance type for the cluster. Must be a valid type as listed in https://databricks.com/product/aws-pricing/instance-types.
        If not specified, a value determined by the Tecton backend is used.
    :param instance_availability: Instance availability for the cluster : "spot", "on_demand", or "spot_with_fallback".
        If not specified, default is spot.
    :param first_on_demand: The first `first_on_demand` nodes of the cluster will use on_demand instances. The rest will use the type specified by instance_availability.
        If first_on_demand >= 1, the driver node use on_demand instance type.
    :param number_of_workers: Number of instances for the materialization job. If not specified, a value determined by the Tecton backend is used
    :param root_volume_size_in_gb: Size of the root volume in GB per instance for the materialization job.
        If not specified, a value determined by the Tecton backend is used.
    :param extra_pip_dependencies: Extra pip dependencies to be installed on the cluster. Can be PyPI packages or S3 wheels/eggs.
    :param spark_config: Map of Spark configuration options and their respective values that will be passed to the
        FeatureView materialization Spark cluster. Currently, we support only the following options: ``spark.driver.memory``, ``spark.driver.memoryOverhead``, ``spark.executor.memory``, ``spark.executor.memoryOverhead``

    Note on ``extra_pip_dependencies``: This is a list of pip package names that will be installed during
    materialization. These libraries will only be available to use inside Spark UDFs. For example, if you set
    ``extra_pip_dependencies=["tensorflow"]``, you can use it in your transformation as shown below.

    An example of DatabricksClusterConfig.

    .. code-block:: python

        from tecton import batch_feature_view, Input, DatabricksClusterConfig

        @batch_feature_view(
            inputs={'credit_scores': Input(credit_scores_batch)},
            # Can be an argument instance to a batch feature view decorator
            batch_cluster_config = DatabricksClusterConfig(
                instance_type = 'm5.2xlarge',
                spark_config = {"spark.executor.memory" : "12g"}
                extra_pip_dependencies=["tensorflow"],
            ),
            # Other named arguments to batch feature view
            ...
        )

        # Use the tensorflow package in the UDF since tensorflow will be installed
        # on the Databricks Spark cluster. The import has to be within the UDF body. Putting it at the
        # top of the file or inside transformation function won't work.

        @transformation(mode='pyspark')
        def test_transformation(transformation_input):
            from pyspark.sql import functions as F
            from pyspark.sql.types import IntegerType

            def my_tensorflow(x):
                import tensorflow as tf
                return int(tf.math.log1p(float(x)).numpy())

            my_tensorflow_udf = F.udf(my_tensorflow, IntegerType())

            return transformation_input.select(
                'entity_id',
                'timestamp',
                my_tensorflow_udf('clicks').alias('log1p_clicks')
            )
    """

    instance_type: Optional[str] = None
    instance_availability: Optional[str] = None
    number_of_workers: Optional[int] = None
    first_on_demand: Optional[int] = None
    root_volume_size_in_gb: Optional[int] = None
    extra_pip_dependencies: Optional[List[str]] = None
    spark_config: Optional[Dict[str, str]] = None

    def _to_proto(self) -> feature_view_pb2.NewDatabricksClusterConfig:
        proto = feature_view_pb2.NewDatabricksClusterConfig()
        if self.instance_type:
            proto.instance_type = self.instance_type
        if self.instance_availability:
            if self.instance_availability not in DATABRICKS_SUPPORTED_AVAILABILITY:
                raise ValueError(
                    f"Instance availability {self.instance_availability} is not supported. Choose {AVAILABILITY_SPOT}, {AVAILABILITY_ON_DEMAND} or {AVAILABILITY_SPOT_FALLBACK}"
                )
            proto.instance_availability = self.instance_availability
        if self.number_of_workers:
            proto.number_of_workers = self.number_of_workers
        if self.root_volume_size_in_gb:
            proto.root_volume_size_in_gb = self.root_volume_size_in_gb
        if self.first_on_demand:
            proto.first_on_demand = self.first_on_demand
        if self.extra_pip_dependencies:
            # Pretty easy to do e.g. extra_pip_dependencies="tensorflow" by mistake and end up with
            # [t, e, n, s, o, r, f, l, o, w] as a list of dependencies passed to the Spark job.
            #
            # Since this is annoying to debug, we check for that here.
            if isinstance(self.extra_pip_dependencies, str):
                raise ValueError("extra_pip_dependencies must be a list")
            proto.extra_pip_dependencies.extend(self.extra_pip_dependencies)
        if self.spark_config:
            spark_config = SparkConfigWrapper(self.spark_config)._to_proto()
            proto.spark_config.CopyFrom(spark_config)

        return proto

    def _to_cluster_proto(self) -> feature_view_pb2.ClusterConfig:
        return feature_view_pb2.ClusterConfig(new_databricks=self._to_proto())


@attr.s(auto_attribs=True)
class SparkConfigWrapper:
    spark_config_map: Dict[str, str]

    SUPPORTED_OPTS = {
        "spark.driver.memory": "spark_driver_memory",
        "spark.executor.memory": "spark_executor_memory",
        "spark.driver.memoryOverhead": "spark_driver_memory_overhead",
        "spark.executor.memoryOverhead": "spark_executor_memory_overhead",
    }

    def _to_proto(self):
        proto = feature_view_pb2.SparkConfig()
        for opt, val in self.spark_config_map.items():
            if opt not in self.SUPPORTED_OPTS:
                raise ValueError(f"Spark config option {opt} is not supported.")
            setattr(proto, self.SUPPORTED_OPTS[opt], val)

        return proto


@attr.s(auto_attribs=True)
class ParquetConfig:
    """(Config Class) ParquetConfig Class.

    This class describes the attributes of Parquet-based offline feature store storage for the feature definition.
    """

    def _to_proto(self):
        store_config = feature_view_pb2.OfflineFeatureStoreConfig()
        store_config.parquet.SetInParent()
        return store_config


@attr.s(auto_attribs=True)
class DeltaConfig:
    """(Config Class) DeltaConfig Class.

    This class describes the attributes of DeltaLake-based offline feature store storage for the feature definition.
    """

    time_partition_size: Optional[str] = "24h"
    """The size of a time partition in the DeltaLake table, specified as a time string. Defaults to "24h"."""

    def _to_proto(self):
        store_config = feature_view_pb2.OfflineFeatureStoreConfig()
        store_config.delta.time_partition_size.FromTimedelta(
            pendulum.duration(seconds=pytimeparse.parse(self.time_partition_size))
        )
        return store_config


@attr.s(auto_attribs=True)
class DynamoConfig:
    """(Config Class) DynamoConfig Class.

    This class describes the attributes of DynamoDB based online feature store for the feature definition.
    Currently there are no attributes for this class.
    Users can specify online_config = DynamoConfig()
    """

    def _to_proto(self):
        store_config = feature_view_pb2.OnlineStoreConfig()
        store_config.dynamo.enabled = True
        store_config.dynamo.SetInParent()
        return store_config


@attr.s(auto_attribs=True)
class RedisConfig:
    """(Config Class) RedisConfig Class.

    This class describes the attributes of Redis based online feature store for the feature definition.
    Currently there are no attributes for this class.
    Users can specify online_config = RedisConfig()
    Note : Your Tecton deployment needs to be connected to Redis before you can use this configuration option.
    Please contact Tecton support for details.
    """

    def _to_proto(self):
        store_config = feature_view_pb2.OnlineStoreConfig()
        store_config.redis.enabled = True
        store_config.redis.SetInParent()
        return store_config


@attr.s(auto_attribs=True)
class MonitoringConfig:
    """Configuration used to specify monitoring options.

    This class describes the FeatureView materialization freshness and alerting configurations. Requires
    materialization to be enabled. Freshness monitoring requires online materialization to be enabled.
    See `Monitoring Materialization`_ for more details.

    :param monitor_freshness: Defines the enabled/disabled state of monitoring when feature data is materialized to the online feature store.
    :type monitor_freshness: bool
    :param expected_feature_freshness: Threshold used to determine if recently materialized feature data is stale.
        Data is stale if ``now - anchor_time(most_recent_feature_value) > expected_feature_freshness``.
        Value must be at least 2 times the feature tile length.
        If not specified, a value determined by the Tecton backend is used
    :type expected_feature_freshness: str, optional
    :param alert_email: Email that alerts for this FeatureView will be sent to.
    :type alert_email: str, optional

    An example declaration of a MonitorConfig

        .. code-block:: python

            from tecton import batch_feature_view, Input, MonitoringConfig
            # For all named arguments to the batch feature view, see docs for details and types.
            @batch_feature_view(
                inputs={'credit_scores': Input(credit_scores_batch)},
                # Can be an argument instance to a batch feature view decorator
                monitoring = MonitoringConfig(
                    monitor_freshness=True,
                    expected_feature_freshness="1w",
                    alert_email="jules@tecton.ai"
                ),
                # Other named arguments
                ...
            )

            # Your batch feature view function
            def credit_batch_feature_view(credit_scores):
              ...

    .. _Monitoring Materialization: https://docs.tecton.ai/v2/overviews/monitoring_materialization.html
    """

    monitor_freshness: bool
    expected_feature_freshness: Optional[str] = None
    alert_email: Optional[str] = None

    def _to_proto(self) -> feature_view_pb2.MonitoringConfig:
        proto = feature_view_pb2.MonitoringConfig()

        if self.expected_feature_freshness:
            proto.expected_feature_freshness.FromTimedelta(
                pendulum.duration(seconds=pytimeparse.parse(self.expected_feature_freshness))
            )

        proto.alert_email = self.alert_email or ""
        proto.monitor_freshness = self.monitor_freshness
        return proto


@attr.s(auto_attribs=True)
class BackfillConfig:
    """Configuration used to specify backfill options.

    This class configures the backfill behavior of a Batch Feature View. Requires
    materialization to be enabled.

    :param mode: Determines whether Tecton batches backfill jobs:
        [``single_batch_schedule_interval_per_job``, ``multiple_batch_schedule_intervals_per_job``]
    """

    mode: str

    def _to_proto(self) -> feature_view_pb2.BackfillConfig:
        proto = feature_view_pb2.BackfillConfig()
        try:
            proto.mode = feature_view_pb2.BackfillConfigMode.Value(
                BACKFILL_CONFIG_MODE_PROTO_PREFIX + self.mode.upper()
            )
        except ValueError:
            # TODO: add BACKFILL_CONFIG_MODE_SINGLE when supported
            raise errors.InvalidBackfillConfigMode(self.mode, [BACKFILL_CONFIG_MODE_MULTIPLE])
        return proto



import sys

import click
from colorama import Fore

from tecton import conf
from tecton import Workspace
from tecton._internals import metadata_service
from tecton._internals.utils import is_live_workspace
from tecton._internals.workspace_utils import PROD_WORKSPACE_NAME_CLIENT
from tecton.cli import printer
from tecton.cli.command import TectonGroup
from tecton.cli.common import get_current_workspace
from tecton.cli.engine import update_tecton_state
from tecton_proto.metadataservice.metadata_service_pb2 import CreateWorkspaceRequest
from tecton_proto.metadataservice.metadata_service_pb2 import DeleteWorkspaceRequest
from tecton_proto.metadataservice.metadata_service_pb2 import GetWorkspaceRequest
from tecton_proto.metadataservice.metadata_service_pb2 import ListWorkspacesRequest


@click.group("workspace", cls=TectonGroup)
def workspace():
    """Manipulate a tecton workspace."""


@workspace.command()
@click.argument("workspace")
def select(workspace):
    """Select WORKSPACE."""
    # validate
    workspace_names = {w.name for w in _list_workspaces()}
    if workspace not in workspace_names:
        printer.safe_print(
            f'Workspace "{workspace}" not found. Run `tecton workspace list` to see list of available workspaces.'
        )
        sys.exit(1)

    _switch_to_workspace(workspace)


@workspace.command()
def list():
    """List available workspaces."""
    current_workspace = get_current_workspace()
    workspaces = _list_workspaces()
    materializable = [w.name for w in workspaces if w.capabilities.materializable]
    nonmaterializable = [w.name for w in workspaces if not w.capabilities.materializable]

    if materializable:
        printer.safe_print("Live Workspaces:")
        for name in materializable:
            marker = "*" if name == current_workspace else " "
            printer.safe_print(f"{marker} {name}")

    # Print whitespace between the two sections if needed.
    if materializable and nonmaterializable:
        printer.safe_print()

    if nonmaterializable:
        printer.safe_print("Development Workspaces:")
        for name in nonmaterializable:
            marker = "*" if name == current_workspace else " "
            printer.safe_print(f"{marker} {name}")


@workspace.command()
def show():
    """Show active workspace."""
    workspace_name = get_current_workspace()
    workspace = _get_workspace(workspace_name)
    workspace_type = "Live" if workspace.capabilities.materializable else "Development"
    printer.safe_print(f"{workspace_name} ({workspace_type})")


@workspace.command()
@click.argument("workspace")
@click.option(
    "--live/--no-live",
    # Kept for backwards compatibility
    "--automatic-materialization-enabled/--automatic-materialization-disabled",
    default=False,
    help="Create a live workspace, which enables materialization and online serving.",
)
def create(workspace, live):
    """Create a new workspace named WORKSPACE."""
    # There is a check for this on the server side too, but we optimistically validate
    # here as well to show a pretty error message.
    workspace_names = {w.name for w in _list_workspaces()}
    if workspace in workspace_names:
        printer.safe_print(f"Workspace {workspace} already exists", file=sys.stderr)
        sys.exit(1)

    # create
    _create_workspace(workspace, live)

    # switch to new workspace
    _switch_to_workspace(workspace)
    printer.safe_print(
        """
You're now on a new, empty workspace. Workspaces isolate their state,
so if you run "tecton plan" Tecton will not see any existing state
for this configuration.
    """
    )


@workspace.command()
@click.argument("workspace")
@click.option("--yes", "-y", is_flag=True)
def delete(workspace, yes):
    """Delete workspace named WORKSPACE."""
    # validate
    if workspace == PROD_WORKSPACE_NAME_CLIENT:
        printer.safe_print(f"Deleting workspace '{PROD_WORKSPACE_NAME_CLIENT}' not allowed.")
        sys.exit(1)

    is_live = is_live_workspace(workspace)

    # confirm deletion
    confirmation = "y" if yes else None
    while confirmation not in ("y", "n", ""):
        confirmation_text = f'Are you sure you want to delete the workspace "{workspace}"? (y/N)'
        if is_live:
            confirmation_text = f"{Fore.RED}Warning{Fore.RESET}: This will delete any materialized data in this workspace.\n{confirmation_text}"
        confirmation = input(confirmation_text).lower().strip()
    if confirmation == "n" or confirmation == "":
        printer.safe_print("Cancelling delete action.")
        sys.exit(1)

    # archive all fcos in the remote state unconditionally.
    # This will need to be updated when workspaces support materialization.
    with Workspace(workspace):
        update_tecton_state(
            objects=[],
            repo_root="",
            repo_files=[],
            apply=True,
            # Set interactive to False to avoid duplicate confirmation.
            # Confirmation of this action is handled above already.
            interactive=False,
            debug=False,
            upgrade_all=False,
        )

    # delete
    _delete_workspace(workspace)

    # switch to prod if deleted current
    if workspace == get_current_workspace():
        _switch_to_workspace(PROD_WORKSPACE_NAME_CLIENT)


def _switch_to_workspace(workspace_name: str):
    conf.set("TECTON_WORKSPACE", workspace_name)
    conf.save_tecton_configs()
    printer.safe_print(f'Switched to workspace "{workspace_name}".')


def _create_workspace(workspace_name: str, materializable: bool):
    request = CreateWorkspaceRequest()
    request.workspace_name = workspace_name
    request.capabilities.materializable = materializable
    metadata_service.instance().CreateWorkspace(request)
    printer.safe_print(f'Created workspace "{workspace_name}".')


def _delete_workspace(workspace_name: str):
    with Workspace(workspace_name):
        request = DeleteWorkspaceRequest()
        request.workspace = workspace_name
        metadata_service.instance().DeleteWorkspace(request)
    printer.safe_print(f'Deleted workspace "{workspace_name}".')


def _get_workspace(workspace_name: str):
    request = GetWorkspaceRequest()
    request.workspace_name = workspace_name
    response = metadata_service.instance().GetWorkspace(request)
    return response.workspace


def _list_workspaces():
    request = ListWorkspacesRequest()
    response = metadata_service.instance().ListWorkspaces(request)
    return response.workspaces



from datetime import datetime
from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Union

import attr
import numpy
import pandas
import pyspark
import pytz

from tecton._internals.sdk_decorators import sdk_public_method
from tecton_spark import conf


def set_pandas_timezone_from_spark(pandas_df):
    """Match pandas timezone to that of Spark, s.t. the timestamps are correctly displayed."""
    from tecton.tecton_context import TectonContext

    tz = TectonContext.get_instance()._spark.conf.get("spark.sql.session.timeZone")
    for col in pandas_df.columns:
        if pandas.core.dtypes.common.is_datetime64_dtype(pandas_df[col]):
            pandas_df[col] = pandas_df[col].dt.tz_localize(pytz.timezone(tz))
            pandas_df[col] = pandas_df[col].dt.tz_convert(pytz.timezone("UTC"))
            pandas_df[col] = pandas_df[col].dt.tz_localize(None)
    return pandas_df


@attr.s(auto_attribs=True)
class FeatureVector(object):
    """
    FeatureVector Class.

    A FeatureVector is a representation of a single feature vector. Usage of a FeatureVector typically involves
    extracting the feature vector using ``to_pandas()``, ``to_dict()``, or ``to_numpy()``.

    """

    _names: List[str]
    _values: List[Union[int, str, bytes, float, list]]
    _effective_times: List[Optional[datetime]]
    slo_info: Optional[Dict[str, str]] = None

    @sdk_public_method
    def to_dict(
        self, return_effective_times: bool = False
    ) -> Dict[str, Union[int, str, bytes, float, list, dict, None]]:
        """Turns vector into a Python dict.

        :param: return_effective_times: Whether to return the effective time of the feature.

        :return: A Python dict.
        """
        if return_effective_times:
            return {
                name: {"value": self._values[i], "effective_time": self._effective_times[i]}
                for i, name in enumerate(self._names)
            }

        return dict(zip(self._names, self._values))

    @sdk_public_method
    def to_pandas(self, return_effective_times: bool = False) -> pandas.DataFrame:
        """Turns vector into a Pandas DataFrame.

        :param: return_effective_times: Whether to return the effective time of the feature as part of the DataFrame.

        :return: A Pandas DataFrame.
        """
        if return_effective_times:
            return pandas.DataFrame(
                list(zip(self._names, self._values, self._effective_times)), columns=["name", "value", "effective_time"]
            )

        return pandas.DataFrame([self._values], columns=self._names)

    @sdk_public_method
    def to_numpy(self, return_effective_times: bool = False) -> numpy.array:
        """Turns vector into a numpy array.

        :param: return_effective_times: Whether to return the effective time of the feature as part of the list.

        :return: A numpy array.
        """
        if return_effective_times:
            return numpy.array([self._values, self._effective_times])

        return numpy.array(self._values)

    def _update(self, other: "FeatureVector"):
        self._names.extend(other._names)
        self._values.extend(other._values)
        self._effective_times.extend(other._effective_times)


@attr.s(auto_attribs=True)
class DataFrame(object):
    """
    A thin wrapper around Pandas and Spark dataframes.
    """

    _spark_df: Optional[pyspark.sql.DataFrame]
    _pandas_df: Optional[pandas.DataFrame]
    # TODO: Change the type to snowflake.snowpark.DataFrame, currently it will
    # fail type checking for our auto generated doc.
    _snowflake_df: Optional[Any] = None

    @sdk_public_method
    def to_spark(self) -> pyspark.sql.DataFrame:
        """Returns data as a Spark DataFrame.

        :return: A Spark DataFrame.
        """
        if self._spark_df is not None:
            return self._spark_df

        from tecton.tecton_context import TectonContext

        tc = TectonContext.get_instance()
        return tc._spark.createDataFrame(self._pandas_df)

    @sdk_public_method
    def to_pandas(self) -> pandas.DataFrame:
        """Returns data as a Pandas DataFrame.

        :return: A Pandas DataFrame.
        """
        if self._pandas_df is not None:
            return self._pandas_df

        assert self._spark_df is not None or self._snowflake_df is not None

        if self._spark_df is not None:
            return set_pandas_timezone_from_spark(self._spark_df.toPandas())

        if self._snowflake_df is not None:
            return self._snowflake_df.toPandas()

    @sdk_public_method
    def to_snowflake(self):
        """Returns data as a Snowflake DataFrame.

        :return: A Snowflake DataFrame.
        :meta private:
        """
        if self._snowflake_df is not None:
            return self._snowflake_df

        assert self._pandas_df is not None

        from tecton.snowflake_context import SnowflakeContext

        if conf.get_or_none("ALPHA_SNOWFLAKE_SNOWPARK_ENABLED"):
            return SnowflakeContext.get_instance().get_session().createDataFrame(self._pandas_df)
        else:
            raise Exception("to_snowflake() is only available with Snowpark enabled")

    @classmethod
    def _create(cls, df: Union[pyspark.sql.DataFrame, pandas.DataFrame]):
        """Creates a Tecton DataFrame from a Spark or Pandas DataFrame."""
        if isinstance(df, pandas.DataFrame):
            return cls(spark_df=None, pandas_df=df, snowflake_df=None)
        elif isinstance(df, pyspark.sql.DataFrame):
            return cls(spark_df=df, pandas_df=None, snowflake_df=None)

        raise TypeError(f"DataFrame must be of type pandas.DataFrame or pyspark.sql.Dataframe, not {type(df)}")

    @classmethod
    # This should be merged into _create once snowpark is installed with pip
    def _create_with_snowflake(cls, df: "snowflake.snowpark.DataFrame"):
        """Creates a Tecton DataFrame from a Snowflake DataFrame."""
        from snowflake.snowpark import DataFrame as SnowflakeDataFrame

        if isinstance(df, SnowflakeDataFrame):
            return cls(spark_df=None, pandas_df=None, snowflake_df=df)

        raise TypeError(f"DataFrame must be of type snowflake.snowpark.Dataframe, not {type(df)}")



# TODO: Remove this once CLI can run independent of PySpark.
try:
    import pyspark
except ImportError:
    raise Exception(
        "PySpark is not available, please install tecton with the [pyspark] extra (e.g. 'tecton[pyspark]==<VERSION NUMBER>')"
    )

#  When adding/removing a class here, also mirror them to sdk/docs/index.rst,
#  or they won't be included in the autogenerated docs.
# nopycln: file
# noreorder
from tecton_spark import conf
from tecton.feature_services.logging_config import LoggingConfig
from tecton.interactive.data_source import get_data_source
from tecton.data_sources.data_source import BatchDataSource
from tecton.data_sources.data_source import StreamDataSource
from tecton.data_sources.datetime_partition_column import DatetimePartitionColumn
from tecton.entities.entity import Entity  # noqa: F401
from tecton_spark.online_serving_index import OnlineServingIndex
from tecton.interactive.entity import get_entity
from tecton.features_common.feature_configs import (
    ExistingClusterConfig,
    FeatureAggregation,
    DeltaConfig,
    ParquetConfig,
    MonitoringConfig,
    DatabricksClusterConfig,
    EMRClusterConfig,
    RedisConfig,
    DynamoConfig,
)
from tecton.feature_services.feature_service import FeatureService, FeaturesConfig
from tecton.interactive.feature_table import get_feature_table  # noqa: F401
from tecton.interactive.feature_view import get_feature_view  # noqa: F401
from tecton.interactive.feature_service import get_feature_service  # noqa: F401
from tecton.interactive.transformation import get_transformation  # noqa: F401
from tecton.interactive.data_frame import DataFrame, FeatureVector  # noqa: F401
from tecton.interactive.workspace import Workspace, get_workspace  # noqa: F401
from tecton.interactive.feature_retrieval import get_historical_features, get_online_features  # noqa: F401
from tecton.interactive.feature_freshness import get_feature_freshness
from tecton_spark.function_serialization import inlined

from tecton_spark.materialization_context import materialization_context
from tecton import version as __version_lib
from tecton.data_sources.hive_data_source import HiveDSConfig
from tecton.data_sources.kafka_data_source import KafkaDSConfig
from tecton.data_sources.kinesis_data_source import KinesisDSConfig
from tecton.data_sources.file_data_source import FileDSConfig
from tecton.data_sources.redshift_data_source import RedshiftDSConfig
from tecton.data_sources.snowflake_data_source import SnowflakeDSConfig
from tecton.interactive.dataset import get_dataset
from tecton.credentials_override import set_credentials
from tecton.fco_listers import (
    list_entities,
    list_transformations,
    list_feature_services,
    list_workspaces,
    list_feature_views,
    list_datasets,
    list_data_sources,
    list_feature_tables,
)
from tecton.legacy_tecton_objects import (
    sql_transformation,
    pyspark_transformation,
    online_transformation,
    DataSourceConfig,
    VirtualDataSource,
    TemporalFeaturePackage,
    TemporalAggregateFeaturePackage,
    OnlineFeaturePackage,
    PushFeaturePackage,
    MaterializationConfig,
    NewEMRClusterConfig,
    NewDatabricksClusterConfig,
)
from tecton_spark.pipeline_helper import RequestDataSource
from tecton.transformations.const import const
from tecton.transformations.transformation import transformation
from tecton.transformations.transformation import SPARK_SQL_MODE
from tecton.transformations.transformation import PYSPARK_MODE
from tecton.transformations.transformation import PANDAS_MODE
from tecton.feature_table.feature_table import FeatureTable
from tecton.feature_views.feature_view import Input

# Import `batch_feature_view` from alpha if Snowflake is enabled.
if conf.get_or_none("ALPHA_SNOWFLAKE_COMPUTE_ENABLED"):
    from tecton._alpha import batch_feature_view
else:
    from tecton.feature_views.feature_view import batch_feature_view
    from tecton.feature_views.feature_view import batch_window_aggregate_feature_view

from tecton.feature_views.feature_view import stream_feature_view
from tecton.feature_views.feature_view import stream_window_aggregate_feature_view
from tecton.feature_views.feature_view import on_demand_feature_view
from tecton.feature_views.feature_view import INPUT_WINDOW_UNBOUNDED_PRECEDING
from tecton.feature_views.feature_view import BackfillConfig
from tecton_spark.time_utils import WINDOW_UNBOUNDED_PRECEDING
from tecton_spark.udfs import tecton_sliding_window_udf
from tecton.transformations.builtin_transformations import tecton_sliding_window

__version__ = __version_lib.get_version()



from tecton._internals.sdk_decorators import sdk_public_method
from tecton_spark import conf
from tecton_spark import logger as logger_lib
from tecton_spark.snowflake.snowflake_utils import supress_snowflake_logs

logger = logger_lib.get_logger("SnowflakeContext")


class SnowflakeContext:
    """
    Get access to Snowflake session.
    """

    _current_context_instance = None
    _session = None
    _connection = None

    def __init__(self):
        connection_parameters = {
            "user": conf.get_or_raise("SNOWFLAKE_USER"),
            "password": conf.get_or_raise("SNOWFLAKE_PASSWORD"),
            "account": conf.get_or_raise("SNOWFLAKE_ACCOUNT_IDENTIFIER"),
            "warehouse": conf.get_or_raise("SNOWFLAKE_WAREHOUSE"),
            # Database and schema are required for Snowpark to create various temporary objects under the covers
            "database": conf.get_or_raise("SNOWFLAKE_DATABASE"),
            "schema": "PUBLIC",
        }
        if conf.get_or_none("ALPHA_SNOWFLAKE_SNOWPARK_ENABLED"):
            from snowflake.snowpark import Session

            supress_snowflake_logs()
            self._session = Session.builder.configs(connection_parameters).create()
        else:
            import snowflake.connector

            supress_snowflake_logs()
            self._connection = snowflake.connector.connect(**connection_parameters)

    def get_session(self):
        if conf.get_or_none("ALPHA_SNOWFLAKE_SNOWPARK_ENABLED"):
            return self._session
        else:
            raise Exception("Snowflake session is only available with Snowpark enabled, use get_connection() instead")

    def get_connection(self):
        if conf.get_or_none("ALPHA_SNOWFLAKE_SNOWPARK_ENABLED"):
            raise Exception(
                "Snowflake connection is only available without Snowpark enabled, use get_session() instead"
            )
        else:
            return self._connection

    @classmethod
    @sdk_public_method
    def get_instance(cls) -> "SnowflakeContext":
        """
        Get the singleton instance of SnowflakeContext.
        """
        # If the instance doesn't exist, creates a new SnowflakeContext from
        # an existing Spark context. Alternatively, creates a new Spark context on the fly.
        if cls._current_context_instance is not None:
            return cls._current_context_instance
        else:
            return cls._generate_and_set_new_instance()

    @classmethod
    def _generate_and_set_new_instance(cls) -> "SnowflakeContext":
        logger.debug(f"Generating new Snowflake session")
        cls._current_context_instance = cls()
        return cls._current_context_instance



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/remote_spark.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import data_source_pb2 as tecton__proto_dot_args_dot_data__source__pb2
from tecton_proto.args import data_source_config_pb2 as tecton__proto_dot_args_dot_data__source__config__pb2
from tecton_proto.args import user_defined_function_pb2 as tecton__proto_dot_args_dot_user__defined__function__pb2
from tecton_proto.args import feature_view_pb2 as tecton__proto_dot_args_dot_feature__view__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.common import schema_pb2 as tecton__proto_dot_common_dot_schema__pb2
from tecton_proto.common import spark_schema_pb2 as tecton__proto_dot_common_dot_spark__schema__pb2
from tecton_proto.data import feature_view_pb2 as tecton__proto_dot_data_dot_feature__view__pb2
from tecton_proto.data import virtual_data_source_pb2 as tecton__proto_dot_data_dot_virtual__data__source__pb2
from tecton_proto.data import new_transformation_pb2 as tecton__proto_dot_data_dot_new__transformation__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/remote_spark.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n$tecton_proto/data/remote_spark.proto\x12\x11tecton_proto.data\x1a#tecton_proto/args/data_source.proto\x1a*tecton_proto/args/data_source_config.proto\x1a-tecton_proto/args/user_defined_function.proto\x1a$tecton_proto/args/feature_view.proto\x1a\x1ctecton_proto/common/id.proto\x1a tecton_proto/common/schema.proto\x1a&tecton_proto/common/spark_schema.proto\x1a$tecton_proto/data/feature_view.proto\x1a+tecton_proto/data/virtual_data_source.proto\x1a*tecton_proto/data/new_transformation.proto\"\xf2\x01\n\x12GetHiveTableSchema\x12\x1a\n\x08\x64\x61tabase\x18\x01 \x01(\tR\x08\x64\x61tabase\x12\x14\n\x05table\x18\x02 \x01(\tR\x05table\x12(\n\x0ftimestampColumn\x18\x03 \x01(\tR\x0ftimestampColumn\x12(\n\x0ftimestampFormat\x18\x04 \x01(\tR\x0ftimestampFormat\x12V\n\x12rawBatchTranslator\x18\x05 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12rawBatchTranslator\"\xd6\x01\n\x16GetRedshiftTableSchema\x12\x1a\n\x08\x65ndpoint\x18\x01 \x01(\tR\x08\x65ndpoint\x12\x14\n\x05table\x18\x02 \x01(\tR\x05table\x12\x14\n\x05query\x18\x03 \x01(\tR\x05query\x12V\n\x12rawBatchTranslator\x18\x04 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12rawBatchTranslator\x12\x1c\n\ttemps3dir\x18\x05 \x01(\tR\ttemps3dir\"f\n\x12GetSnowflakeSchema\x12P\n\rsnowflakeArgs\x18\x01 \x01(\x0b\x32*.tecton_proto.args.SnowflakeDataSourceArgsR\rsnowflakeArgs\"\x8d\x03\n\x13GetFileSourceSchema\x12\x10\n\x03uri\x18\x01 \x01(\tR\x03uri\x12\x1e\n\nfileFormat\x18\x02 \x01(\tR\nfileFormat\x12\x30\n\x13\x63onvertToGlueFormat\x18\x03 \x01(\x08R\x13\x63onvertToGlueFormat\x12V\n\x12rawBatchTranslator\x18\x04 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12rawBatchTranslator\x12\x1c\n\tschemaUri\x18\x05 \x01(\tR\tschemaUri\x12(\n\x0ftimestampColumn\x18\x06 \x01(\tR\x0ftimestampColumn\x12(\n\x0ftimestampFormat\x18\x07 \x01(\tR\x0ftimestampFormat\x12H\n\x0eschemaOverride\x18\x08 \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x0eschemaOverride\"\x92\x01\n\x16GetKinesisSourceSchema\x12\x1e\n\nstreamName\x18\x01 \x01(\tR\nstreamName\x12X\n\x13rawStreamTranslator\x18\x02 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x13rawStreamTranslator\"\xea\x01\n\x14GetKafkaSourceSchema\x12X\n\x13rawStreamTranslator\x18\x01 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x13rawStreamTranslator\x12\x32\n\x15ssl_keystore_location\x18\x02 \x01(\tR\x13sslKeystoreLocation\x12\x44\n\x1fssl_keystore_password_secret_id\x18\x03 \x01(\tR\x1bsslKeystorePasswordSecretId\"\xb4\x03\n#GetFeatureViewMaterializationSchema\x12V\n\x14virtual_data_sources\x18\x02 \x03(\x0b\x32$.tecton_proto.data.VirtualDataSourceR\x12virtualDataSources\x12N\n\x0ftransformations\x18\x03 \x03(\x0b\x32$.tecton_proto.data.NewTransformationR\x0ftransformations\x12#\n\rtimestamp_key\x18\x04 \x01(\tR\x0ctimestampKey\x12\x1b\n\tjoin_keys\x18\x05 \x03(\tR\x08joinKeys\x12V\n\x12temporal_aggregate\x18\x06 \x01(\x0b\x32\'.tecton_proto.data.NewTemporalAggregateR\x11temporalAggregate\x12\x45\n\x0c\x66\x65\x61ture_view\x18\x07 \x01(\x0b\x32\".tecton_proto.args.FeatureViewArgsR\x0b\x66\x65\x61tureViewJ\x04\x08\x01\x10\x02\"\x91\x02\n\x14GetFeatureViewSchema\x12V\n\x14virtual_data_sources\x18\x02 \x03(\x0b\x32$.tecton_proto.data.VirtualDataSourceR\x12virtualDataSources\x12N\n\x0ftransformations\x18\x03 \x03(\x0b\x32$.tecton_proto.data.NewTransformationR\x0ftransformations\x12\x45\n\x0c\x66\x65\x61ture_view\x18\x05 \x01(\x0b\x32\".tecton_proto.args.FeatureViewArgsR\x0b\x66\x65\x61tureViewJ\x04\x08\x01\x10\x02J\x04\x08\x04\x10\x05\"\x8c\x02\n\x15GetQueryPlanInfoForFV\x12V\n\x14virtual_data_sources\x18\x02 \x03(\x0b\x32$.tecton_proto.data.VirtualDataSourceR\x12virtualDataSources\x12N\n\x0ftransformations\x18\x03 \x03(\x0b\x32$.tecton_proto.data.NewTransformationR\x0ftransformations\x12\x45\n\x0c\x66\x65\x61ture_view\x18\x04 \x01(\x0b\x32\".tecton_proto.args.FeatureViewArgsR\x0b\x66\x65\x61tureViewJ\x04\x08\x01\x10\x02\"\xd0\x08\n\x0e\x45xecuteRequest\x12W\n\x12getHiveTableSchema\x18\x01 \x01(\x0b\x32%.tecton_proto.data.GetHiveTableSchemaH\x00R\x12getHiveTableSchema\x12\x63\n\x16getRedshiftTableSchema\x18\x02 \x01(\x0b\x32).tecton_proto.data.GetRedshiftTableSchemaH\x00R\x16getRedshiftTableSchema\x12Z\n\x13getFileSourceSchema\x18\x03 \x01(\x0b\x32&.tecton_proto.data.GetFileSourceSchemaH\x00R\x13getFileSourceSchema\x12\x63\n\x16getKinesisSourceSchema\x18\x04 \x01(\x0b\x32).tecton_proto.data.GetKinesisSourceSchemaH\x00R\x16getKinesisSourceSchema\x12]\n\x14getKafkaSourceSchema\x18\x05 \x01(\x0b\x32\'.tecton_proto.data.GetKafkaSourceSchemaH\x00R\x14getKafkaSourceSchema\x12]\n\x14getFeatureViewSchema\x18\x0e \x01(\x0b\x32\'.tecton_proto.data.GetFeatureViewSchemaH\x00R\x14getFeatureViewSchema\x12\x8a\x01\n#getFeatureViewMaterializationSchema\x18\x0f \x01(\x0b\x32\x36.tecton_proto.data.GetFeatureViewMaterializationSchemaH\x00R#getFeatureViewMaterializationSchema\x12W\n\x12getSnowflakeSchema\x18\x0c \x01(\x0b\x32%.tecton_proto.data.GetSnowflakeSchemaH\x00R\x12getSnowflakeSchema\x12`\n\x15getQueryPlanInfoForFV\x18\r \x01(\x0b\x32(.tecton_proto.data.GetQueryPlanInfoForFVH\x00R\x15getQueryPlanInfoForFV\x12H\n\x07\x65nvVars\x18\x10 \x03(\x0b\x32..tecton_proto.data.ExecuteRequest.EnvVarsEntryR\x07\x65nvVars\x1a:\n\x0c\x45nvVarsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\t\n\x07requestJ\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\x08\x10\tJ\x04\x08\t\x10\nJ\x04\x08\n\x10\x0bJ\x04\x08\x0b\x10\x0cJ\x04\x08\x11\x10\x12\"x\n\rQueryPlanInfo\x12\x1b\n\thas_joins\x18\x01 \x01(\x08R\x08hasJoins\x12)\n\x10has_aggregations\x18\x02 \x01(\x08R\x0fhasAggregations\x12\x1f\n\x0btable_names\x18\x03 \x03(\tR\ntableNames\"\xb4\x02\n\rExecuteResult\x12&\n\runcaughtError\x18\x01 \x01(\tH\x00R\runcaughtError\x12*\n\x0fvalidationError\x18\x05 \x01(\tH\x00R\x0fvalidationError\x12\x44\n\x0bsparkSchema\x18\x02 \x01(\x0b\x32 .tecton_proto.common.SparkSchemaH\x00R\x0bsparkSchema\x12H\n\rqueryPlanInfo\x18\x03 \x01(\x0b\x32 .tecton_proto.data.QueryPlanInfoH\x00R\rqueryPlanInfo\x12\x35\n\x06schema\x18\x04 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaH\x00R\x06schemaB\x08\n\x06resultB\x13\n\x0f\x63om.tecton.dataP\x01'
  ,
  dependencies=[tecton__proto_dot_args_dot_data__source__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_data__source__config__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_user__defined__function__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_feature__view__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_schema__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_spark__schema__pb2.DESCRIPTOR,tecton__proto_dot_data_dot_feature__view__pb2.DESCRIPTOR,tecton__proto_dot_data_dot_virtual__data__source__pb2.DESCRIPTOR,tecton__proto_dot_data_dot_new__transformation__pb2.DESCRIPTOR,])




_GETHIVETABLESCHEMA = _descriptor.Descriptor(
  name='GetHiveTableSchema',
  full_name='tecton_proto.data.GetHiveTableSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='tecton_proto.data.GetHiveTableSchema.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='database', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table', full_name='tecton_proto.data.GetHiveTableSchema.table', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='table', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestampColumn', full_name='tecton_proto.data.GetHiveTableSchema.timestampColumn', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampColumn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestampFormat', full_name='tecton_proto.data.GetHiveTableSchema.timestampFormat', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampFormat', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rawBatchTranslator', full_name='tecton_proto.data.GetHiveTableSchema.rawBatchTranslator', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawBatchTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=457,
  serialized_end=699,
)


_GETREDSHIFTTABLESCHEMA = _descriptor.Descriptor(
  name='GetRedshiftTableSchema',
  full_name='tecton_proto.data.GetRedshiftTableSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='tecton_proto.data.GetRedshiftTableSchema.endpoint', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='endpoint', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table', full_name='tecton_proto.data.GetRedshiftTableSchema.table', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='table', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='query', full_name='tecton_proto.data.GetRedshiftTableSchema.query', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='query', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rawBatchTranslator', full_name='tecton_proto.data.GetRedshiftTableSchema.rawBatchTranslator', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawBatchTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='temps3dir', full_name='tecton_proto.data.GetRedshiftTableSchema.temps3dir', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='temps3dir', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=702,
  serialized_end=916,
)


_GETSNOWFLAKESCHEMA = _descriptor.Descriptor(
  name='GetSnowflakeSchema',
  full_name='tecton_proto.data.GetSnowflakeSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='snowflakeArgs', full_name='tecton_proto.data.GetSnowflakeSchema.snowflakeArgs', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='snowflakeArgs', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=918,
  serialized_end=1020,
)


_GETFILESOURCESCHEMA = _descriptor.Descriptor(
  name='GetFileSourceSchema',
  full_name='tecton_proto.data.GetFileSourceSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uri', full_name='tecton_proto.data.GetFileSourceSchema.uri', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='uri', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fileFormat', full_name='tecton_proto.data.GetFileSourceSchema.fileFormat', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='fileFormat', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='convertToGlueFormat', full_name='tecton_proto.data.GetFileSourceSchema.convertToGlueFormat', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='convertToGlueFormat', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rawBatchTranslator', full_name='tecton_proto.data.GetFileSourceSchema.rawBatchTranslator', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawBatchTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schemaUri', full_name='tecton_proto.data.GetFileSourceSchema.schemaUri', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaUri', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestampColumn', full_name='tecton_proto.data.GetFileSourceSchema.timestampColumn', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampColumn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestampFormat', full_name='tecton_proto.data.GetFileSourceSchema.timestampFormat', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampFormat', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schemaOverride', full_name='tecton_proto.data.GetFileSourceSchema.schemaOverride', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schemaOverride', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1023,
  serialized_end=1420,
)


_GETKINESISSOURCESCHEMA = _descriptor.Descriptor(
  name='GetKinesisSourceSchema',
  full_name='tecton_proto.data.GetKinesisSourceSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='streamName', full_name='tecton_proto.data.GetKinesisSourceSchema.streamName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='streamName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rawStreamTranslator', full_name='tecton_proto.data.GetKinesisSourceSchema.rawStreamTranslator', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawStreamTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1423,
  serialized_end=1569,
)


_GETKAFKASOURCESCHEMA = _descriptor.Descriptor(
  name='GetKafkaSourceSchema',
  full_name='tecton_proto.data.GetKafkaSourceSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='rawStreamTranslator', full_name='tecton_proto.data.GetKafkaSourceSchema.rawStreamTranslator', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawStreamTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssl_keystore_location', full_name='tecton_proto.data.GetKafkaSourceSchema.ssl_keystore_location', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sslKeystoreLocation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssl_keystore_password_secret_id', full_name='tecton_proto.data.GetKafkaSourceSchema.ssl_keystore_password_secret_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sslKeystorePasswordSecretId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1572,
  serialized_end=1806,
)


_GETFEATUREVIEWMATERIALIZATIONSCHEMA = _descriptor.Descriptor(
  name='GetFeatureViewMaterializationSchema',
  full_name='tecton_proto.data.GetFeatureViewMaterializationSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='virtual_data_sources', full_name='tecton_proto.data.GetFeatureViewMaterializationSchema.virtual_data_sources', index=0,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='virtualDataSources', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transformations', full_name='tecton_proto.data.GetFeatureViewMaterializationSchema.transformations', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='transformations', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_key', full_name='tecton_proto.data.GetFeatureViewMaterializationSchema.timestamp_key', index=2,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampKey', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='join_keys', full_name='tecton_proto.data.GetFeatureViewMaterializationSchema.join_keys', index=3,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='joinKeys', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='temporal_aggregate', full_name='tecton_proto.data.GetFeatureViewMaterializationSchema.temporal_aggregate', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='temporalAggregate', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view', full_name='tecton_proto.data.GetFeatureViewMaterializationSchema.feature_view', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureView', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1809,
  serialized_end=2245,
)


_GETFEATUREVIEWSCHEMA = _descriptor.Descriptor(
  name='GetFeatureViewSchema',
  full_name='tecton_proto.data.GetFeatureViewSchema',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='virtual_data_sources', full_name='tecton_proto.data.GetFeatureViewSchema.virtual_data_sources', index=0,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='virtualDataSources', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transformations', full_name='tecton_proto.data.GetFeatureViewSchema.transformations', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='transformations', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view', full_name='tecton_proto.data.GetFeatureViewSchema.feature_view', index=2,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureView', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2248,
  serialized_end=2521,
)


_GETQUERYPLANINFOFORFV = _descriptor.Descriptor(
  name='GetQueryPlanInfoForFV',
  full_name='tecton_proto.data.GetQueryPlanInfoForFV',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='virtual_data_sources', full_name='tecton_proto.data.GetQueryPlanInfoForFV.virtual_data_sources', index=0,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='virtualDataSources', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='transformations', full_name='tecton_proto.data.GetQueryPlanInfoForFV.transformations', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='transformations', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view', full_name='tecton_proto.data.GetQueryPlanInfoForFV.feature_view', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureView', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2524,
  serialized_end=2792,
)


_EXECUTEREQUEST_ENVVARSENTRY = _descriptor.Descriptor(
  name='EnvVarsEntry',
  full_name='tecton_proto.data.ExecuteRequest.EnvVarsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='tecton_proto.data.ExecuteRequest.EnvVarsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='tecton_proto.data.ExecuteRequest.EnvVarsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3788,
  serialized_end=3846,
)

_EXECUTEREQUEST = _descriptor.Descriptor(
  name='ExecuteRequest',
  full_name='tecton_proto.data.ExecuteRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='getHiveTableSchema', full_name='tecton_proto.data.ExecuteRequest.getHiveTableSchema', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='getHiveTableSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='getRedshiftTableSchema', full_name='tecton_proto.data.ExecuteRequest.getRedshiftTableSchema', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='getRedshiftTableSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='getFileSourceSchema', full_name='tecton_proto.data.ExecuteRequest.getFileSourceSchema', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='getFileSourceSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='getKinesisSourceSchema', full_name='tecton_proto.data.ExecuteRequest.getKinesisSourceSchema', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='getKinesisSourceSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='getKafkaSourceSchema', full_name='tecton_proto.data.ExecuteRequest.getKafkaSourceSchema', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='getKafkaSourceSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='getFeatureViewSchema', full_name='tecton_proto.data.ExecuteRequest.getFeatureViewSchema', index=5,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='getFeatureViewSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='getFeatureViewMaterializationSchema', full_name='tecton_proto.data.ExecuteRequest.getFeatureViewMaterializationSchema', index=6,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='getFeatureViewMaterializationSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='getSnowflakeSchema', full_name='tecton_proto.data.ExecuteRequest.getSnowflakeSchema', index=7,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='getSnowflakeSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='getQueryPlanInfoForFV', full_name='tecton_proto.data.ExecuteRequest.getQueryPlanInfoForFV', index=8,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='getQueryPlanInfoForFV', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='envVars', full_name='tecton_proto.data.ExecuteRequest.envVars', index=9,
      number=16, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='envVars', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_EXECUTEREQUEST_ENVVARSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='request', full_name='tecton_proto.data.ExecuteRequest.request',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=2795,
  serialized_end=3899,
)


_QUERYPLANINFO = _descriptor.Descriptor(
  name='QueryPlanInfo',
  full_name='tecton_proto.data.QueryPlanInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='has_joins', full_name='tecton_proto.data.QueryPlanInfo.has_joins', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='hasJoins', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='has_aggregations', full_name='tecton_proto.data.QueryPlanInfo.has_aggregations', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='hasAggregations', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_names', full_name='tecton_proto.data.QueryPlanInfo.table_names', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='tableNames', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3901,
  serialized_end=4021,
)


_EXECUTERESULT = _descriptor.Descriptor(
  name='ExecuteResult',
  full_name='tecton_proto.data.ExecuteResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uncaughtError', full_name='tecton_proto.data.ExecuteResult.uncaughtError', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='uncaughtError', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='validationError', full_name='tecton_proto.data.ExecuteResult.validationError', index=1,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='validationError', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sparkSchema', full_name='tecton_proto.data.ExecuteResult.sparkSchema', index=2,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sparkSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='queryPlanInfo', full_name='tecton_proto.data.ExecuteResult.queryPlanInfo', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='queryPlanInfo', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schema', full_name='tecton_proto.data.ExecuteResult.schema', index=4,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='result', full_name='tecton_proto.data.ExecuteResult.result',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=4024,
  serialized_end=4332,
)

_GETHIVETABLESCHEMA.fields_by_name['rawBatchTranslator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_GETREDSHIFTTABLESCHEMA.fields_by_name['rawBatchTranslator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_GETSNOWFLAKESCHEMA.fields_by_name['snowflakeArgs'].message_type = tecton__proto_dot_args_dot_data__source__pb2._SNOWFLAKEDATASOURCEARGS
_GETFILESOURCESCHEMA.fields_by_name['rawBatchTranslator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_GETFILESOURCESCHEMA.fields_by_name['schemaOverride'].message_type = tecton__proto_dot_common_dot_spark__schema__pb2._SPARKSCHEMA
_GETKINESISSOURCESCHEMA.fields_by_name['rawStreamTranslator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_GETKAFKASOURCESCHEMA.fields_by_name['rawStreamTranslator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_GETFEATUREVIEWMATERIALIZATIONSCHEMA.fields_by_name['virtual_data_sources'].message_type = tecton__proto_dot_data_dot_virtual__data__source__pb2._VIRTUALDATASOURCE
_GETFEATUREVIEWMATERIALIZATIONSCHEMA.fields_by_name['transformations'].message_type = tecton__proto_dot_data_dot_new__transformation__pb2._NEWTRANSFORMATION
_GETFEATUREVIEWMATERIALIZATIONSCHEMA.fields_by_name['temporal_aggregate'].message_type = tecton__proto_dot_data_dot_feature__view__pb2._NEWTEMPORALAGGREGATE
_GETFEATUREVIEWMATERIALIZATIONSCHEMA.fields_by_name['feature_view'].message_type = tecton__proto_dot_args_dot_feature__view__pb2._FEATUREVIEWARGS
_GETFEATUREVIEWSCHEMA.fields_by_name['virtual_data_sources'].message_type = tecton__proto_dot_data_dot_virtual__data__source__pb2._VIRTUALDATASOURCE
_GETFEATUREVIEWSCHEMA.fields_by_name['transformations'].message_type = tecton__proto_dot_data_dot_new__transformation__pb2._NEWTRANSFORMATION
_GETFEATUREVIEWSCHEMA.fields_by_name['feature_view'].message_type = tecton__proto_dot_args_dot_feature__view__pb2._FEATUREVIEWARGS
_GETQUERYPLANINFOFORFV.fields_by_name['virtual_data_sources'].message_type = tecton__proto_dot_data_dot_virtual__data__source__pb2._VIRTUALDATASOURCE
_GETQUERYPLANINFOFORFV.fields_by_name['transformations'].message_type = tecton__proto_dot_data_dot_new__transformation__pb2._NEWTRANSFORMATION
_GETQUERYPLANINFOFORFV.fields_by_name['feature_view'].message_type = tecton__proto_dot_args_dot_feature__view__pb2._FEATUREVIEWARGS
_EXECUTEREQUEST_ENVVARSENTRY.containing_type = _EXECUTEREQUEST
_EXECUTEREQUEST.fields_by_name['getHiveTableSchema'].message_type = _GETHIVETABLESCHEMA
_EXECUTEREQUEST.fields_by_name['getRedshiftTableSchema'].message_type = _GETREDSHIFTTABLESCHEMA
_EXECUTEREQUEST.fields_by_name['getFileSourceSchema'].message_type = _GETFILESOURCESCHEMA
_EXECUTEREQUEST.fields_by_name['getKinesisSourceSchema'].message_type = _GETKINESISSOURCESCHEMA
_EXECUTEREQUEST.fields_by_name['getKafkaSourceSchema'].message_type = _GETKAFKASOURCESCHEMA
_EXECUTEREQUEST.fields_by_name['getFeatureViewSchema'].message_type = _GETFEATUREVIEWSCHEMA
_EXECUTEREQUEST.fields_by_name['getFeatureViewMaterializationSchema'].message_type = _GETFEATUREVIEWMATERIALIZATIONSCHEMA
_EXECUTEREQUEST.fields_by_name['getSnowflakeSchema'].message_type = _GETSNOWFLAKESCHEMA
_EXECUTEREQUEST.fields_by_name['getQueryPlanInfoForFV'].message_type = _GETQUERYPLANINFOFORFV
_EXECUTEREQUEST.fields_by_name['envVars'].message_type = _EXECUTEREQUEST_ENVVARSENTRY
_EXECUTEREQUEST.oneofs_by_name['request'].fields.append(
  _EXECUTEREQUEST.fields_by_name['getHiveTableSchema'])
_EXECUTEREQUEST.fields_by_name['getHiveTableSchema'].containing_oneof = _EXECUTEREQUEST.oneofs_by_name['request']
_EXECUTEREQUEST.oneofs_by_name['request'].fields.append(
  _EXECUTEREQUEST.fields_by_name['getRedshiftTableSchema'])
_EXECUTEREQUEST.fields_by_name['getRedshiftTableSchema'].containing_oneof = _EXECUTEREQUEST.oneofs_by_name['request']
_EXECUTEREQUEST.oneofs_by_name['request'].fields.append(
  _EXECUTEREQUEST.fields_by_name['getFileSourceSchema'])
_EXECUTEREQUEST.fields_by_name['getFileSourceSchema'].containing_oneof = _EXECUTEREQUEST.oneofs_by_name['request']
_EXECUTEREQUEST.oneofs_by_name['request'].fields.append(
  _EXECUTEREQUEST.fields_by_name['getKinesisSourceSchema'])
_EXECUTEREQUEST.fields_by_name['getKinesisSourceSchema'].containing_oneof = _EXECUTEREQUEST.oneofs_by_name['request']
_EXECUTEREQUEST.oneofs_by_name['request'].fields.append(
  _EXECUTEREQUEST.fields_by_name['getKafkaSourceSchema'])
_EXECUTEREQUEST.fields_by_name['getKafkaSourceSchema'].containing_oneof = _EXECUTEREQUEST.oneofs_by_name['request']
_EXECUTEREQUEST.oneofs_by_name['request'].fields.append(
  _EXECUTEREQUEST.fields_by_name['getFeatureViewSchema'])
_EXECUTEREQUEST.fields_by_name['getFeatureViewSchema'].containing_oneof = _EXECUTEREQUEST.oneofs_by_name['request']
_EXECUTEREQUEST.oneofs_by_name['request'].fields.append(
  _EXECUTEREQUEST.fields_by_name['getFeatureViewMaterializationSchema'])
_EXECUTEREQUEST.fields_by_name['getFeatureViewMaterializationSchema'].containing_oneof = _EXECUTEREQUEST.oneofs_by_name['request']
_EXECUTEREQUEST.oneofs_by_name['request'].fields.append(
  _EXECUTEREQUEST.fields_by_name['getSnowflakeSchema'])
_EXECUTEREQUEST.fields_by_name['getSnowflakeSchema'].containing_oneof = _EXECUTEREQUEST.oneofs_by_name['request']
_EXECUTEREQUEST.oneofs_by_name['request'].fields.append(
  _EXECUTEREQUEST.fields_by_name['getQueryPlanInfoForFV'])
_EXECUTEREQUEST.fields_by_name['getQueryPlanInfoForFV'].containing_oneof = _EXECUTEREQUEST.oneofs_by_name['request']
_EXECUTERESULT.fields_by_name['sparkSchema'].message_type = tecton__proto_dot_common_dot_spark__schema__pb2._SPARKSCHEMA
_EXECUTERESULT.fields_by_name['queryPlanInfo'].message_type = _QUERYPLANINFO
_EXECUTERESULT.fields_by_name['schema'].message_type = tecton__proto_dot_common_dot_schema__pb2._SCHEMA
_EXECUTERESULT.oneofs_by_name['result'].fields.append(
  _EXECUTERESULT.fields_by_name['uncaughtError'])
_EXECUTERESULT.fields_by_name['uncaughtError'].containing_oneof = _EXECUTERESULT.oneofs_by_name['result']
_EXECUTERESULT.oneofs_by_name['result'].fields.append(
  _EXECUTERESULT.fields_by_name['validationError'])
_EXECUTERESULT.fields_by_name['validationError'].containing_oneof = _EXECUTERESULT.oneofs_by_name['result']
_EXECUTERESULT.oneofs_by_name['result'].fields.append(
  _EXECUTERESULT.fields_by_name['sparkSchema'])
_EXECUTERESULT.fields_by_name['sparkSchema'].containing_oneof = _EXECUTERESULT.oneofs_by_name['result']
_EXECUTERESULT.oneofs_by_name['result'].fields.append(
  _EXECUTERESULT.fields_by_name['queryPlanInfo'])
_EXECUTERESULT.fields_by_name['queryPlanInfo'].containing_oneof = _EXECUTERESULT.oneofs_by_name['result']
_EXECUTERESULT.oneofs_by_name['result'].fields.append(
  _EXECUTERESULT.fields_by_name['schema'])
_EXECUTERESULT.fields_by_name['schema'].containing_oneof = _EXECUTERESULT.oneofs_by_name['result']
DESCRIPTOR.message_types_by_name['GetHiveTableSchema'] = _GETHIVETABLESCHEMA
DESCRIPTOR.message_types_by_name['GetRedshiftTableSchema'] = _GETREDSHIFTTABLESCHEMA
DESCRIPTOR.message_types_by_name['GetSnowflakeSchema'] = _GETSNOWFLAKESCHEMA
DESCRIPTOR.message_types_by_name['GetFileSourceSchema'] = _GETFILESOURCESCHEMA
DESCRIPTOR.message_types_by_name['GetKinesisSourceSchema'] = _GETKINESISSOURCESCHEMA
DESCRIPTOR.message_types_by_name['GetKafkaSourceSchema'] = _GETKAFKASOURCESCHEMA
DESCRIPTOR.message_types_by_name['GetFeatureViewMaterializationSchema'] = _GETFEATUREVIEWMATERIALIZATIONSCHEMA
DESCRIPTOR.message_types_by_name['GetFeatureViewSchema'] = _GETFEATUREVIEWSCHEMA
DESCRIPTOR.message_types_by_name['GetQueryPlanInfoForFV'] = _GETQUERYPLANINFOFORFV
DESCRIPTOR.message_types_by_name['ExecuteRequest'] = _EXECUTEREQUEST
DESCRIPTOR.message_types_by_name['QueryPlanInfo'] = _QUERYPLANINFO
DESCRIPTOR.message_types_by_name['ExecuteResult'] = _EXECUTERESULT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetHiveTableSchema = _reflection.GeneratedProtocolMessageType('GetHiveTableSchema', (_message.Message,), {
  'DESCRIPTOR' : _GETHIVETABLESCHEMA,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.GetHiveTableSchema)
  })
_sym_db.RegisterMessage(GetHiveTableSchema)

GetRedshiftTableSchema = _reflection.GeneratedProtocolMessageType('GetRedshiftTableSchema', (_message.Message,), {
  'DESCRIPTOR' : _GETREDSHIFTTABLESCHEMA,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.GetRedshiftTableSchema)
  })
_sym_db.RegisterMessage(GetRedshiftTableSchema)

GetSnowflakeSchema = _reflection.GeneratedProtocolMessageType('GetSnowflakeSchema', (_message.Message,), {
  'DESCRIPTOR' : _GETSNOWFLAKESCHEMA,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.GetSnowflakeSchema)
  })
_sym_db.RegisterMessage(GetSnowflakeSchema)

GetFileSourceSchema = _reflection.GeneratedProtocolMessageType('GetFileSourceSchema', (_message.Message,), {
  'DESCRIPTOR' : _GETFILESOURCESCHEMA,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.GetFileSourceSchema)
  })
_sym_db.RegisterMessage(GetFileSourceSchema)

GetKinesisSourceSchema = _reflection.GeneratedProtocolMessageType('GetKinesisSourceSchema', (_message.Message,), {
  'DESCRIPTOR' : _GETKINESISSOURCESCHEMA,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.GetKinesisSourceSchema)
  })
_sym_db.RegisterMessage(GetKinesisSourceSchema)

GetKafkaSourceSchema = _reflection.GeneratedProtocolMessageType('GetKafkaSourceSchema', (_message.Message,), {
  'DESCRIPTOR' : _GETKAFKASOURCESCHEMA,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.GetKafkaSourceSchema)
  })
_sym_db.RegisterMessage(GetKafkaSourceSchema)

GetFeatureViewMaterializationSchema = _reflection.GeneratedProtocolMessageType('GetFeatureViewMaterializationSchema', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATUREVIEWMATERIALIZATIONSCHEMA,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.GetFeatureViewMaterializationSchema)
  })
_sym_db.RegisterMessage(GetFeatureViewMaterializationSchema)

GetFeatureViewSchema = _reflection.GeneratedProtocolMessageType('GetFeatureViewSchema', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATUREVIEWSCHEMA,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.GetFeatureViewSchema)
  })
_sym_db.RegisterMessage(GetFeatureViewSchema)

GetQueryPlanInfoForFV = _reflection.GeneratedProtocolMessageType('GetQueryPlanInfoForFV', (_message.Message,), {
  'DESCRIPTOR' : _GETQUERYPLANINFOFORFV,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.GetQueryPlanInfoForFV)
  })
_sym_db.RegisterMessage(GetQueryPlanInfoForFV)

ExecuteRequest = _reflection.GeneratedProtocolMessageType('ExecuteRequest', (_message.Message,), {

  'EnvVarsEntry' : _reflection.GeneratedProtocolMessageType('EnvVarsEntry', (_message.Message,), {
    'DESCRIPTOR' : _EXECUTEREQUEST_ENVVARSENTRY,
    '__module__' : 'tecton_proto.data.remote_spark_pb2'
    # @@protoc_insertion_point(class_scope:tecton_proto.data.ExecuteRequest.EnvVarsEntry)
    })
  ,
  'DESCRIPTOR' : _EXECUTEREQUEST,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.ExecuteRequest)
  })
_sym_db.RegisterMessage(ExecuteRequest)
_sym_db.RegisterMessage(ExecuteRequest.EnvVarsEntry)

QueryPlanInfo = _reflection.GeneratedProtocolMessageType('QueryPlanInfo', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPLANINFO,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.QueryPlanInfo)
  })
_sym_db.RegisterMessage(QueryPlanInfo)

ExecuteResult = _reflection.GeneratedProtocolMessageType('ExecuteResult', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTERESULT,
  '__module__' : 'tecton_proto.data.remote_spark_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.ExecuteResult)
  })
_sym_db.RegisterMessage(ExecuteResult)


DESCRIPTOR._options = None
_EXECUTEREQUEST_ENVVARSENTRY._options = None
# @@protoc_insertion_point(module_scope)



from typing import Optional

from tecton.data_sources.base_data_source_config import BaseBatchDSConfig
from tecton_proto.args import data_source_pb2
from tecton_proto.args import virtual_data_source_pb2
from tecton_spark import function_serialization


class RedshiftDSConfig(BaseBatchDSConfig):
    """
    Configuration used to reference a Redshift table or query.

    The RedshiftDSConfig class is used to create a reference to a Redshift table. You can also create a
    reference to a query on one or more tables, which will be registered in Tecton in a similar way as a view
    is registered in other data systems.

    This class used as an input to a :class:`BatchDataSource`'s parameter ``batch_ds_config``. This class is not
    a Tecton Object: it is a grouping of parameters. Declaring this class alone will not register a data source.
    Instead, declare as part of ``BatchDataSource`` that takes this configuration class instance as a parameter.
    """

    def __init__(
        self,
        endpoint: str,
        table: Optional[str] = None,
        raw_batch_translator=None,
        temp_s3: Optional[str] = None,
        query: Optional[str] = None,
        timestamp_key: Optional[str] = None,
    ):
        """
        Instantiates a new RedshiftDSConfig. One of table and query should be specified when creating this file.

        :param endpoint: The connection endpoint to Redshift
                         (e.g. redshift-cluster-1.cigcwzsdltjs.us-west-2.redshift.amazonaws.com:5439/dev).
        :param table: The Redshift table for this Data source. Only one of table and query should be specified.
        :param raw_batch_translator: Python user defined function f(DataFrame) -> DataFrame that takes in raw
                                     PySpark data source DataFrame and translates it to the DataFrame to be
                                     consumed by the Feature View. See an example of
                                     raw_batch_translator in the `User Guide`_.
        :param query: A Redshift query for this Data source. Only one of table and query should be specified.
        :param temp_s3: [deprecated] An S3 URI destination for intermediate data that is needed for Redshift.
                        (e.g. s3://tecton-ai-test-cluster-redshift-data)
        :param timestamp_key: (Optional) The name of the timestamp column (after the raw_batch_translator has been applied).
                               The column name does not need to be specified if there is exactly one timestamp column after the translator is applied.
                               This is needed for efficient time filtering when materializing batch features.

        :return: A RedshiftDSConfig class instance.

        .. _User Guide: https://docs.tecton.ai/v2/overviews/framework/data_sources.html

        Example of a RedshiftDSConfig declaration:

        .. code-block:: python

            from tecton import RedshiftDSConfig

            # Declare RedshiftDSConfig instance object that can be used as an argument in BatchDataSource
            redshift_ds_config = RedshiftDSConfig(endpoint="cluster-1.us-west-2.redshift.amazonaws.com:5439/dev",
                                                  table="ad_serving_features",
                                                  query="SELECT timestamp as ts, created, user_id, ad_id, duration"
                                                        "FROM ad_serving_features")
        """
        self._args = args = data_source_pb2.RedshiftDataSourceArgs()
        args.endpoint = endpoint

        if table and query:
            raise AssertionError(f"Should only specify one of table and query sources for redshift")
        if not table and not query:
            raise AssertionError(f"Missing both table and query sources for redshift, exactly one must be present")

        if table:
            args.table = table
        else:
            args.query = query

        if raw_batch_translator is not None:
            args.raw_batch_translator.CopyFrom(function_serialization.to_proto(raw_batch_translator))
        if timestamp_key:
            args.timestamp_key = timestamp_key

    def _merge_batch_args(self, data_source_args: virtual_data_source_pb2.VirtualDataSourceArgs):
        data_source_args.redshift_ds_config.CopyFrom(self._args)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/stream_data_source.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import spark_schema_pb2 as tecton__proto_dot_common_dot_spark__schema__pb2
from tecton_proto.args import user_defined_function_pb2 as tecton__proto_dot_args_dot_user__defined__function__pb2
from tecton_proto.args import data_source_config_pb2 as tecton__proto_dot_args_dot_data__source__config__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/stream_data_source.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n*tecton_proto/data/stream_data_source.proto\x12\x11tecton_proto.data\x1a&tecton_proto/common/spark_schema.proto\x1a-tecton_proto/args/user_defined_function.proto\x1a*tecton_proto/args/data_source_config.proto\"\\\n\x0fKinesisDSConfig\x12\x1f\n\x0bstream_name\x18\x01 \x01(\tR\nstreamName\x12\x16\n\x06region\x18\x02 \x01(\tR\x06regionJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\"\xfd\x02\n\rKafkaDSConfig\x12+\n\x11\x62ootstrap_servers\x18\x01 \x01(\tR\x10\x62ootstrapServers\x12\x16\n\x06topics\x18\x02 \x01(\tR\x06topics\x12\x32\n\x15ssl_keystore_location\x18\x07 \x01(\tR\x13sslKeystoreLocation\x12\x44\n\x1fssl_keystore_password_secret_id\x18\x08 \x01(\tR\x1bsslKeystorePasswordSecretId\x12\x36\n\x17ssl_truststore_location\x18\t \x01(\tR\x15sslTruststoreLocation\x12H\n!ssl_truststore_password_secret_id\x18\n \x01(\tR\x1dsslTruststorePasswordSecretId\x12+\n\x11security_protocol\x18\x0b \x01(\tR\x10securityProtocol\"\x99\x05\n\x10StreamDataSource\x12T\n\x13kinesis_data_source\x18\x01 \x01(\x0b\x32\".tecton_proto.data.KinesisDSConfigH\x00R\x11kinesisDataSource\x12N\n\x11kafka_data_source\x18\x0b \x01(\x0b\x32 .tecton_proto.data.KafkaDSConfigH\x00R\x0fkafkaDataSource\x12\x43\n\x0cspark_schema\x18\t \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x0bsparkSchema\x12Z\n\x15raw_stream_translator\x18\x03 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x13rawStreamTranslator\x12<\n\x1a\x64\x65\x64uplication_column_names\x18\n \x03(\tR\x18\x64\x65\x64uplicationColumnNames\x12\x1f\n\x0btime_column\x18\x06 \x01(\tR\ntimeColumn\x12\x44\n\rstream_config\x18\x07 \x01(\x0b\x32\x1f.tecton_proto.args.StreamConfigR\x0cstreamConfig\x12\x44\n\x07options\x18\x08 \x03(\x0b\x32*.tecton_proto.data.StreamDataSource.OptionR\x07options\x1a\x30\n\x06Option\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05valueB\x0f\n\rstream_sourceJ\x04\x08\x02\x10\x03J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\x42\x13\n\x0f\x63om.tecton.dataP\x01'
  ,
  dependencies=[tecton__proto_dot_common_dot_spark__schema__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_user__defined__function__pb2.DESCRIPTOR,tecton__proto_dot_args_dot_data__source__config__pb2.DESCRIPTOR,])




_KINESISDSCONFIG = _descriptor.Descriptor(
  name='KinesisDSConfig',
  full_name='tecton_proto.data.KinesisDSConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='stream_name', full_name='tecton_proto.data.KinesisDSConfig.stream_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='streamName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='region', full_name='tecton_proto.data.KinesisDSConfig.region', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='region', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=196,
  serialized_end=288,
)


_KAFKADSCONFIG = _descriptor.Descriptor(
  name='KafkaDSConfig',
  full_name='tecton_proto.data.KafkaDSConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bootstrap_servers', full_name='tecton_proto.data.KafkaDSConfig.bootstrap_servers', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='bootstrapServers', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='topics', full_name='tecton_proto.data.KafkaDSConfig.topics', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='topics', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssl_keystore_location', full_name='tecton_proto.data.KafkaDSConfig.ssl_keystore_location', index=2,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sslKeystoreLocation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssl_keystore_password_secret_id', full_name='tecton_proto.data.KafkaDSConfig.ssl_keystore_password_secret_id', index=3,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sslKeystorePasswordSecretId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssl_truststore_location', full_name='tecton_proto.data.KafkaDSConfig.ssl_truststore_location', index=4,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sslTruststoreLocation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ssl_truststore_password_secret_id', full_name='tecton_proto.data.KafkaDSConfig.ssl_truststore_password_secret_id', index=5,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sslTruststorePasswordSecretId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='security_protocol', full_name='tecton_proto.data.KafkaDSConfig.security_protocol', index=6,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='securityProtocol', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=291,
  serialized_end=672,
)


_STREAMDATASOURCE_OPTION = _descriptor.Descriptor(
  name='Option',
  full_name='tecton_proto.data.StreamDataSource.Option',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='tecton_proto.data.StreamDataSource.Option.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='key', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='tecton_proto.data.StreamDataSource.Option.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='value', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1257,
  serialized_end=1305,
)

_STREAMDATASOURCE = _descriptor.Descriptor(
  name='StreamDataSource',
  full_name='tecton_proto.data.StreamDataSource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='kinesis_data_source', full_name='tecton_proto.data.StreamDataSource.kinesis_data_source', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='kinesisDataSource', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kafka_data_source', full_name='tecton_proto.data.StreamDataSource.kafka_data_source', index=1,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='kafkaDataSource', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='spark_schema', full_name='tecton_proto.data.StreamDataSource.spark_schema', index=2,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sparkSchema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='raw_stream_translator', full_name='tecton_proto.data.StreamDataSource.raw_stream_translator', index=3,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='rawStreamTranslator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deduplication_column_names', full_name='tecton_proto.data.StreamDataSource.deduplication_column_names', index=4,
      number=10, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='deduplicationColumnNames', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_column', full_name='tecton_proto.data.StreamDataSource.time_column', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timeColumn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stream_config', full_name='tecton_proto.data.StreamDataSource.stream_config', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='streamConfig', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='options', full_name='tecton_proto.data.StreamDataSource.options', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='options', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_STREAMDATASOURCE_OPTION, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='stream_source', full_name='tecton_proto.data.StreamDataSource.stream_source',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=675,
  serialized_end=1340,
)

_STREAMDATASOURCE_OPTION.containing_type = _STREAMDATASOURCE
_STREAMDATASOURCE.fields_by_name['kinesis_data_source'].message_type = _KINESISDSCONFIG
_STREAMDATASOURCE.fields_by_name['kafka_data_source'].message_type = _KAFKADSCONFIG
_STREAMDATASOURCE.fields_by_name['spark_schema'].message_type = tecton__proto_dot_common_dot_spark__schema__pb2._SPARKSCHEMA
_STREAMDATASOURCE.fields_by_name['raw_stream_translator'].message_type = tecton__proto_dot_args_dot_user__defined__function__pb2._USERDEFINEDFUNCTION
_STREAMDATASOURCE.fields_by_name['stream_config'].message_type = tecton__proto_dot_args_dot_data__source__config__pb2._STREAMCONFIG
_STREAMDATASOURCE.fields_by_name['options'].message_type = _STREAMDATASOURCE_OPTION
_STREAMDATASOURCE.oneofs_by_name['stream_source'].fields.append(
  _STREAMDATASOURCE.fields_by_name['kinesis_data_source'])
_STREAMDATASOURCE.fields_by_name['kinesis_data_source'].containing_oneof = _STREAMDATASOURCE.oneofs_by_name['stream_source']
_STREAMDATASOURCE.oneofs_by_name['stream_source'].fields.append(
  _STREAMDATASOURCE.fields_by_name['kafka_data_source'])
_STREAMDATASOURCE.fields_by_name['kafka_data_source'].containing_oneof = _STREAMDATASOURCE.oneofs_by_name['stream_source']
DESCRIPTOR.message_types_by_name['KinesisDSConfig'] = _KINESISDSCONFIG
DESCRIPTOR.message_types_by_name['KafkaDSConfig'] = _KAFKADSCONFIG
DESCRIPTOR.message_types_by_name['StreamDataSource'] = _STREAMDATASOURCE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

KinesisDSConfig = _reflection.GeneratedProtocolMessageType('KinesisDSConfig', (_message.Message,), {
  'DESCRIPTOR' : _KINESISDSCONFIG,
  '__module__' : 'tecton_proto.data.stream_data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.KinesisDSConfig)
  })
_sym_db.RegisterMessage(KinesisDSConfig)

KafkaDSConfig = _reflection.GeneratedProtocolMessageType('KafkaDSConfig', (_message.Message,), {
  'DESCRIPTOR' : _KAFKADSCONFIG,
  '__module__' : 'tecton_proto.data.stream_data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.KafkaDSConfig)
  })
_sym_db.RegisterMessage(KafkaDSConfig)

StreamDataSource = _reflection.GeneratedProtocolMessageType('StreamDataSource', (_message.Message,), {

  'Option' : _reflection.GeneratedProtocolMessageType('Option', (_message.Message,), {
    'DESCRIPTOR' : _STREAMDATASOURCE_OPTION,
    '__module__' : 'tecton_proto.data.stream_data_source_pb2'
    # @@protoc_insertion_point(class_scope:tecton_proto.data.StreamDataSource.Option)
    })
  ,
  'DESCRIPTOR' : _STREAMDATASOURCE,
  '__module__' : 'tecton_proto.data.stream_data_source_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.StreamDataSource)
  })
_sym_db.RegisterMessage(StreamDataSource)
_sym_db.RegisterMessage(StreamDataSource.Option)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
